@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package kubernetes
 
 import (
@@ -9,10 +12,9 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
+	"strconv"
 	"strings"
 
-	"github.com/hashicorp/terraform/internal/states/remote"
-	"github.com/hashicorp/terraform/internal/states/statemgr"
 	k8serrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
@@ -21,6 +23,9 @@ import (
 	_ "k8s.io/client-go/plugin/pkg/client/auth" // Import to initialize client auth plugins.
 	"k8s.io/utils/pointer"
 
+	"github.com/hashicorp/terraform/internal/states/remote"
+	"github.com/hashicorp/terraform/internal/states/statemgr"
+
 	coordinationv1 "k8s.io/api/coordination/v1"
 	coordinationclientv1 "k8s.io/client-go/kubernetes/typed/coordination/v1"
 )
@@ -30,7 +35,10 @@ const (
 	tfstateSecretSuffixKey    = "tfstateSecretSuffix"
 	tfstateWorkspaceKey       = "tfstateWorkspace"
 	tfstateLockInfoAnnotation = "app.terraform.io/lock-info"
-	managedByKey              = "app.kubernetes.io/managed-by"
+
+	managedByKey = "app.kubernetes.io/managed-by"
+
+	defaultChunkSize = 1048576
 )
 
 type RemoteClient struct {
@@ -43,34 +51,32 @@ type RemoteClient struct {
 }
 
 func (c *RemoteClient) Get() (payload *remote.Payload, err error) {
-	secretName, err := c.createSecretName()
+	secretList, err := c.getSecrets()
 	if err != nil {
 		return nil, err
 	}
-	secret, err := c.kubernetesSecretClient.Get(context.Background(), secretName, metav1.GetOptions{})
-	if err != nil {
-		if k8serrors.IsNotFound(err) {
-			return nil, nil
-		}
-		return nil, err
-	}
 
-	secretData := getSecretData(secret)
-	stateRaw, ok := secretData[tfstateKey]
-	if !ok {
-		// The secret exists but there is no state in it
+	if len(secretList) == 0 {
 		return nil, nil
 	}
 
-	stateRawString := stateRaw.(string)
+	var data []string
+	for _, secret := range secretList {
+		secretData := getSecretData(&secret)
+		stateRaw, ok := secretData[tfstateKey]
+		if !ok {
+			// The secret exists but there is no state in it
+			return nil, nil
+		}
+		data = append(data, stateRaw.(string))
+	}
 
-	state, err := uncompressState(stateRawString)
+	state, err := uncompressState(data)
 	if err != nil {
 		return nil, err
 	}
 
 	md5 := md5.Sum(state)
-
 	p := &remote.Payload{
 		Data: state,
 		MD5:  md5[:],
@@ -78,58 +84,133 @@ func (c *RemoteClient) Get() (payload *remote.Payload, err error) {
 	return p, nil
 }
 
+func (c *RemoteClient) getSecrets() ([]unstructured.Unstructured, error) {
+	ls := metav1.SetAsLabelSelector(c.getLabels())
+	res, err := c.kubernetesSecretClient.List(context.Background(),
+		metav1.ListOptions{
+			LabelSelector: metav1.FormatLabelSelector(ls),
+		},
+	)
+	if err != nil {
+		return []unstructured.Unstructured{}, err
+	}
+
+	// NOTE we need to sort the list as the k8s API will return
+	// the list sorted by name which will corrupt the state when
+	// the number of secrets is greater than 10
+	items := make([]unstructured.Unstructured, len(res.Items))
+	for _, item := range res.Items {
+		name := item.GetName()
+		nameParts := strings.Split(name, "-")
+		index, err := strconv.Atoi(nameParts[len(nameParts)-1])
+		if err != nil {
+			index = 0
+		}
+		items[index] = item
+	}
+	return items, nil
+}
+
 func (c *RemoteClient) Put(data []byte) error {
 	ctx := context.Background()
-	secretName, err := c.createSecretName()
-	if err != nil {
-		return err
-	}
 
 	payload, err := compressState(data)
 	if err != nil {
 		return err
 	}
 
-	secret, err := c.getSecret(secretName)
+	chunks := chunkPayload(payload, defaultChunkSize)
+	existingSecrets, err := c.getSecrets()
 	if err != nil {
-		if !k8serrors.IsNotFound(err) {
+		return err
+	}
+
+	for idx, data := range chunks {
+		secretName, err := c.createSecretName(idx)
+		if err != nil {
 			return err
 		}
 
-		secret = &unstructured.Unstructured{
-			Object: map[string]interface{}{
-				"metadata": metav1.ObjectMeta{
-					Name:        secretName,
-					Namespace:   c.namespace,
-					Labels:      c.getLabels(),
-					Annotations: map[string]string{"encoding": "gzip"},
+		secret, err := c.getSecret(secretName)
+		if err != nil {
+			if !k8serrors.IsNotFound(err) {
+				return err
+			}
+			secret = &unstructured.Unstructured{
+				Object: map[string]interface{}{
+					"metadata": metav1.ObjectMeta{
+						Name:        secretName,
+						Namespace:   c.namespace,
+						Labels:      c.getLabels(),
+						Annotations: map[string]string{"encoding": "gzip"},
+					},
 				},
-			},
+			}
+			secret, err = c.kubernetesSecretClient.Create(ctx, secret, metav1.CreateOptions{})
+			if err != nil {
+				return err
+			}
 		}
 
-		secret, err = c.kubernetesSecretClient.Create(ctx, secret, metav1.CreateOptions{})
+		setState(secret, data)
+		_, err = c.kubernetesSecretClient.Update(ctx, secret, metav1.UpdateOptions{})
 		if err != nil {
 			return err
 		}
 	}
 
-	setState(secret, payload)
-	_, err = c.kubernetesSecretClient.Update(ctx, secret, metav1.UpdateOptions{})
-	return err
+	// remove old secrets
+	existingSecretCount := len(existingSecrets)
+	newSecretCount := len(chunks)
+	if existingSecretCount == newSecretCount {
+		return nil
+	}
+	for i := newSecretCount; i < existingSecretCount; i++ {
+		secretName, err := c.createSecretName(i)
+		if err != nil {
+			return err
+		}
+		err = c.deleteSecret(secretName)
+		if err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
+// chunkPayload splits the state payload into byte arrays of the given size
+func chunkPayload(buf []byte, size int) [][]byte {
+	chunks := make([][]byte, 0, len(buf)/size+1)
+	for len(buf) >= size {
+		var chunk []byte
+		chunk, buf = buf[:size], buf[size:]
+		chunks = append(chunks, chunk)
+	}
+	if len(buf) > 0 {
+		chunks = append(chunks, buf)
+	}
+	return chunks
 }
 
 // Delete the state secret
 func (c *RemoteClient) Delete() error {
-	secretName, err := c.createSecretName()
+	secretList, err := c.getSecrets()
 	if err != nil {
 		return err
 	}
 
-	err = c.deleteSecret(secretName)
-	if err != nil {
-		if !k8serrors.IsNotFound(err) {
+	for i, _ := range secretList {
+		secretName, err := c.createSecretName(i)
+		if err != nil {
 			return err
 		}
+
+		err = c.deleteSecret(secretName)
+		if err != nil {
+			if !k8serrors.IsNotFound(err) {
+				return err
+			}
+		}
 	}
 
 	leaseName, err := c.createLeaseName()
@@ -320,9 +401,13 @@ func (c *RemoteClient) deleteLease(name string) error {
 	return c.kubernetesLeaseClient.Delete(context.Background(), name, delOps)
 }
 
-func (c *RemoteClient) createSecretName() (string, error) {
+func (c *RemoteClient) createSecretName(idx int) (string, error) {
 	secretName := strings.Join([]string{tfstateKey, c.workspace, c.nameSuffix}, "-")
 
+	if idx > 0 {
+		secretName = fmt.Sprintf("%s-part-%d", secretName, idx)
+	}
+
 	errs := validation.IsDNS1123Subdomain(secretName)
 	if len(errs) > 0 {
 		k8sInfo := `
@@ -336,7 +421,7 @@ The workspace name and key must adhere to Kubernetes naming conventions.`
 }
 
 func (c *RemoteClient) createLeaseName() (string, error) {
-	n, err := c.createSecretName()
+	n, err := c.createSecretName(0)
 	if err != nil {
 		return "", err
 	}
@@ -355,14 +440,18 @@ func compressState(data []byte) ([]byte, error) {
 	return b.Bytes(), nil
 }
 
-func uncompressState(data string) ([]byte, error) {
-	decode, err := base64.StdEncoding.DecodeString(data)
-	if err != nil {
-		return nil, err
+func uncompressState(data []string) ([]byte, error) {
+	var rawData []byte
+	for _, chunk := range data {
+		decode, err := base64.StdEncoding.DecodeString(chunk)
+		if err != nil {
+			return nil, err
+		}
+		rawData = append(rawData, decode...)
 	}
 
 	b := new(bytes.Buffer)
-	gz, err := gzip.NewReader(bytes.NewReader(decode))
+	gz, err := gzip.NewReader(bytes.NewReader(rawData))
 	if err != nil {
 		return nil, err
 	}
