@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package oss
 
 import (
@@ -180,23 +183,22 @@ func (c *RemoteClient) Lock(info *statemgr.LockInfo) (string, error) {
 		},
 	}
 
-	log.Printf("[DEBUG] Recording state lock in tablestore: %#v", putParams)
+	log.Printf("[DEBUG] Recording state lock in tablestore: %#v; LOCKID:%s", putParams, c.lockPath())
 
 	_, err := c.otsClient.PutRow(&tablestore.PutRowRequest{
 		PutRowChange: putParams,
 	})
 	if err != nil {
-		log.Printf("[WARN] Error storing state lock in tablestore: %#v", err)
+		err = fmt.Errorf("invoking PutRow got an error: %#v", err)
 		lockInfo, infoErr := c.getLockInfo()
 		if infoErr != nil {
-			log.Printf("[WARN] Error getting lock info: %#v", err)
-			err = multierror.Append(err, infoErr)
+			err = multierror.Append(err, fmt.Errorf("\ngetting lock info got an error: %#v", infoErr))
 		}
 		lockErr := &statemgr.LockError{
 			Err:  err,
 			Info: lockInfo,
 		}
-		log.Printf("[WARN] state lock error: %#v", lockErr)
+		log.Printf("[ERROR] state lock error: %s", lockErr.Error())
 		return "", lockErr
 	}
 
@@ -386,13 +388,11 @@ func (c *RemoteClient) Unlock(id string) error {
 				},
 			},
 			Condition: &tablestore.RowCondition{
-				RowExistenceExpectation: tablestore.RowExistenceExpectation_EXPECT_EXIST,
+				RowExistenceExpectation: tablestore.RowExistenceExpectation_IGNORE,
 			},
 		},
 	}
 
-	log.Printf("[DEBUG] Deleting state lock from tablestore: %#v", params)
-
 	_, err = c.otsClient.DeleteRow(params)
 
 	if err != nil {
