@@ -1,30 +1,33 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package cloud
 
 import (
 	"bufio"
+	"bytes"
 	"context"
+	"encoding/json"
 	"errors"
 	"fmt"
 	"io"
 	"math"
+	"net/http"
+	"net/url"
 	"strconv"
 	"strings"
 	"time"
 
+	"github.com/hashicorp/go-retryablehttp"
 	tfe "github.com/hashicorp/go-tfe"
+	"github.com/hashicorp/jsonapi"
 	"github.com/hashicorp/terraform/internal/backend"
+	"github.com/hashicorp/terraform/internal/command/jsonformat"
+	"github.com/hashicorp/terraform/internal/logging"
 	"github.com/hashicorp/terraform/internal/plans"
 	"github.com/hashicorp/terraform/internal/terraform"
 )
 
-var (
-	errApplyDiscarded   = errors.New("Apply discarded.")
-	errDestroyDiscarded = errors.New("Destroy discarded.")
-	errRunApproved      = errors.New("approved using the UI or API")
-	errRunDiscarded     = errors.New("discarded using the UI or API")
-	errRunOverridden    = errors.New("overridden using the UI or API")
-)
-
 var (
 	backoffMin = 1000.0
 	backoffMax = 3000.0
@@ -109,7 +112,7 @@ func (b *Cloud) waitForRun(stopCtx, cancelCtx context.Context, op *backend.Opera
 			// Skip checking the workspace queue when we are the current run.
 			if w.CurrentRun == nil || w.CurrentRun.ID != r.ID {
 				found := false
-				options := tfe.RunListOptions{}
+				options := &tfe.RunListOptions{}
 			runlist:
 				for {
 					rl, err := b.client.Runs.List(stopCtx, w.ID, options)
@@ -164,10 +167,10 @@ func (b *Cloud) waitForRun(stopCtx, cancelCtx context.Context, op *backend.Opera
 				}
 			}
 
-			options := tfe.RunQueueOptions{}
+			options := tfe.ReadRunQueueOptions{}
 		search:
 			for {
-				rq, err := b.client.Organizations.RunQueue(stopCtx, b.organization, options)
+				rq, err := b.client.Organizations.ReadRunQueue(stopCtx, b.organization, options)
 				if err != nil {
 					return r, generalError("Failed to retrieve queue", err)
 				}
@@ -190,7 +193,7 @@ func (b *Cloud) waitForRun(stopCtx, cancelCtx context.Context, op *backend.Opera
 			}
 
 			if position > 0 {
-				c, err := b.client.Organizations.Capacity(stopCtx, b.organization)
+				c, err := b.client.Organizations.ReadCapacity(stopCtx, b.organization)
 				if err != nil {
 					return r, generalError("Failed to retrieve capacity", err)
 				}
@@ -208,12 +211,23 @@ func (b *Cloud) waitForRun(stopCtx, cancelCtx context.Context, op *backend.Opera
 	}
 }
 
+func (b *Cloud) waitTaskStage(stopCtx, cancelCtx context.Context, op *backend.Operation, r *tfe.Run, stageID string, outputTitle string) error {
+	integration := &IntegrationContext{
+		B:             b,
+		StopContext:   stopCtx,
+		CancelContext: cancelCtx,
+		Op:            op,
+		Run:           r,
+	}
+	return b.runTaskStage(integration, integration.BeginOutput(outputTitle), stageID)
+}
+
 func (b *Cloud) costEstimate(stopCtx, cancelCtx context.Context, op *backend.Operation, r *tfe.Run) error {
 	if r.CostEstimate == nil {
 		return nil
 	}
 
-	msgPrefix := "Cost estimation"
+	msgPrefix := "Cost Estimation"
 	started := time.Now()
 	updated := started
 	for i := 0; ; i++ {
@@ -260,7 +274,7 @@ func (b *Cloud) costEstimate(stopCtx, cancelCtx context.Context, op *backend.Ope
 			deltaRepr := strings.Replace(ce.DeltaMonthlyCost, "-", "", 1)
 
 			if b.CLI != nil {
-				b.CLI.Output(b.Colorize().Color(msgPrefix + ":\n"))
+				b.CLI.Output(b.Colorize().Color("[bold]" + msgPrefix + ":\n"))
 				b.CLI.Output(b.Colorize().Color(fmt.Sprintf("Resources: %d of %d estimated", ce.MatchedResourcesCount, ce.ResourcesCount)))
 				b.CLI.Output(b.Colorize().Color(fmt.Sprintf("           $%s/mo %s$%s", ce.ProposedMonthlyCost, sign, deltaRepr)))
 
@@ -282,12 +296,12 @@ func (b *Cloud) costEstimate(stopCtx, cancelCtx context.Context, op *backend.Ope
 					elapsed = fmt.Sprintf(
 						" (%s elapsed)", current.Sub(started).Truncate(30*time.Second))
 				}
-				b.CLI.Output(b.Colorize().Color(msgPrefix + ":\n"))
+				b.CLI.Output(b.Colorize().Color("[bold]" + msgPrefix + ":\n"))
 				b.CLI.Output(b.Colorize().Color("Waiting for cost estimate to complete..." + elapsed + "\n"))
 			}
 			continue
 		case tfe.CostEstimateSkippedDueToTargeting:
-			b.CLI.Output(b.Colorize().Color(msgPrefix + ":\n"))
+			b.CLI.Output(b.Colorize().Color("[bold]" + msgPrefix + ":\n"))
 			b.CLI.Output("Not available for this plan, because it was created with the -target option.")
 			b.CLI.Output("\n------------------------------------------------------------------------")
 			return nil
@@ -334,15 +348,15 @@ func (b *Cloud) checkPolicy(stopCtx, cancelCtx context.Context, op *backend.Oper
 		var msgPrefix string
 		switch pc.Scope {
 		case tfe.PolicyScopeOrganization:
-			msgPrefix = "Organization policy check"
+			msgPrefix = "Organization Policy Check"
 		case tfe.PolicyScopeWorkspace:
-			msgPrefix = "Workspace policy check"
+			msgPrefix = "Workspace Policy Check"
 		default:
 			msgPrefix = fmt.Sprintf("Unknown policy check (%s)", pc.Scope)
 		}
 
 		if b.CLI != nil {
-			b.CLI.Output(b.Colorize().Color(msgPrefix + ":\n"))
+			b.CLI.Output(b.Colorize().Color("[bold]" + msgPrefix + ":\n"))
 		}
 
 		if b.CLI != nil {
@@ -377,7 +391,7 @@ func (b *Cloud) checkPolicy(stopCtx, cancelCtx context.Context, op *backend.Oper
 		case tfe.PolicyHardFailed:
 			return fmt.Errorf(msgPrefix + " hard failed.")
 		case tfe.PolicySoftFailed:
-			runUrl := fmt.Sprintf(runHeader, b.hostname, b.organization, op.Workspace, r.ID)
+			runUrl := fmt.Sprintf(runHeader, b.Hostname, b.organization, op.Workspace, r.ID)
 
 			if op.Type == backend.OperationTypePlan || op.UIOut == nil || op.UIIn == nil ||
 				!pc.Actions.IsOverridable || !pc.Permissions.CanOverride {
@@ -388,6 +402,8 @@ func (b *Cloud) checkPolicy(stopCtx, cancelCtx context.Context, op *backend.Oper
 				if _, err = b.client.PolicyChecks.Override(stopCtx, pc.ID); err != nil {
 					return generalError(fmt.Sprintf("Failed to override policy check.\n%s", runUrl), err)
 				}
+			} else if !b.input {
+				return errPolicyOverrideNeedsUIConfirmation
 			} else {
 				opts := &terraform.InputOpts{
 					Id:          "override",
@@ -446,7 +462,7 @@ func (b *Cloud) confirm(stopCtx context.Context, op *backend.Operation, opts *te
 
 				switch keyword {
 				case "override":
-					if r.Status != tfe.RunPolicyOverride {
+					if r.Status != tfe.RunPolicyOverride && r.Status != tfe.RunPostPlanAwaitingDecision {
 						if r.Status == tfe.RunDiscarded {
 							err = errRunDiscarded
 						} else {
@@ -534,3 +550,97 @@ func (b *Cloud) confirm(stopCtx context.Context, op *backend.Operation, opts *te
 
 	return <-result
 }
+
+// This method will fetch the redacted plan output as a byte slice, mirroring
+// the behavior of the similar client.Plans.ReadJSONOutput method.
+//
+// Note: Apologies for the lengthy definition, this is a result of not being
+// able to mock receiver methods
+var readRedactedPlan func(context.Context, url.URL, string, string) ([]byte, error) = func(ctx context.Context, baseURL url.URL, token string, planID string) ([]byte, error) {
+	client := retryablehttp.NewClient()
+	client.RetryMax = 10
+	client.RetryWaitMin = 100 * time.Millisecond
+	client.RetryWaitMax = 400 * time.Millisecond
+	client.Logger = logging.HCLogger()
+
+	u, err := baseURL.Parse(fmt.Sprintf(
+		"plans/%s/json-output-redacted", url.QueryEscape(planID)))
+	if err != nil {
+		return nil, err
+	}
+
+	req, err := retryablehttp.NewRequest("GET", u.String(), nil)
+	if err != nil {
+		return nil, err
+	}
+
+	req.Header.Set("Authorization", "Bearer "+token)
+	req.Header.Set("Accept", "application/json")
+
+	resp, err := client.Do(req)
+	if err != nil {
+		return nil, err
+	}
+	defer resp.Body.Close()
+
+	if err = checkResponseCode(resp); err != nil {
+		return nil, err
+	}
+
+	return io.ReadAll(resp.Body)
+}
+
+// decodeRedactedPlan unmarshals a downloaded redacted plan into a struct the
+// jsonformat.Renderer expects.
+func decodeRedactedPlan(jsonBytes []byte) (*jsonformat.Plan, error) {
+	r := bytes.NewReader(jsonBytes)
+	p := &jsonformat.Plan{}
+	if err := json.NewDecoder(r).Decode(p); err != nil {
+		return nil, err
+	}
+	return p, nil
+}
+
+func checkResponseCode(r *http.Response) error {
+	if r.StatusCode >= 200 && r.StatusCode <= 299 {
+		return nil
+	}
+
+	var errs []string
+	var err error
+
+	switch r.StatusCode {
+	case 401:
+		return tfe.ErrUnauthorized
+	case 404:
+		return tfe.ErrResourceNotFound
+	}
+
+	errs, err = decodeErrorPayload(r)
+	if err != nil {
+		return err
+	}
+
+	return errors.New(strings.Join(errs, "\n"))
+}
+
+func decodeErrorPayload(r *http.Response) ([]string, error) {
+	// Decode the error payload.
+	var errs []string
+	errPayload := &jsonapi.ErrorsPayload{}
+	err := json.NewDecoder(r.Body).Decode(errPayload)
+	if err != nil || len(errPayload.Errors) == 0 {
+		return errs, errors.New(r.Status)
+	}
+
+	// Parse and format the errors.
+	for _, e := range errPayload.Errors {
+		if e.Detail == "" {
+			errs = append(errs, e.Title)
+		} else {
+			errs = append(errs, fmt.Sprintf("%s\n\n%s", e.Title, e.Detail))
+		}
+	}
+
+	return errs, nil
+}
