@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package main
 
 import (
@@ -165,8 +168,8 @@ func Test_migrate_remote_backend_single_org(t *testing.T) {
 				},
 			},
 			validations: func(t *testing.T, orgName string) {
-				wsList, err := tfeClient.Workspaces.List(ctx, orgName, tfe.WorkspaceListOptions{
-					Tags: tfe.String("app"),
+				wsList, err := tfeClient.Workspaces.List(ctx, orgName, &tfe.WorkspaceListOptions{
+					Tags: "app",
 				})
 				if err != nil {
 					t.Fatal(err)
@@ -294,7 +297,7 @@ func Test_migrate_remote_backend_single_org(t *testing.T) {
 				if ws == nil {
 					t.Fatalf("Expected workspace %s to be present, but is not.", expectedName)
 				}
-				wsList, err := tfeClient.Workspaces.List(ctx, orgName, tfe.WorkspaceListOptions{})
+				wsList, err := tfeClient.Workspaces.List(ctx, orgName, nil)
 				if err != nil {
 					t.Fatal(err)
 				}
@@ -428,8 +431,8 @@ func Test_migrate_remote_backend_single_org(t *testing.T) {
 				},
 			},
 			validations: func(t *testing.T, orgName string) {
-				wsList, err := tfeClient.Workspaces.List(ctx, orgName, tfe.WorkspaceListOptions{
-					Tags: tfe.String("app"),
+				wsList, err := tfeClient.Workspaces.List(ctx, orgName, &tfe.WorkspaceListOptions{
+					Tags: "app",
 				})
 				if err != nil {
 					t.Fatal(err)
