@@ -1,14 +1,20 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package jsonplan
 
 import (
 	"encoding/json"
 	"fmt"
 	"sort"
+	"strings"
+	"time"
 
 	"github.com/zclconf/go-cty/cty"
 	ctyjson "github.com/zclconf/go-cty/cty/json"
 
 	"github.com/hashicorp/terraform/internal/addrs"
+	"github.com/hashicorp/terraform/internal/command/jsonchecks"
 	"github.com/hashicorp/terraform/internal/command/jsonconfig"
 	"github.com/hashicorp/terraform/internal/command/jsonstate"
 	"github.com/hashicorp/terraform/internal/configs"
@@ -22,9 +28,25 @@ import (
 // FormatVersion represents the version of the json format and will be
 // incremented for any change to this format that requires changes to a
 // consuming parser.
-const FormatVersion = "1.0"
+const (
+	FormatVersion = "1.2"
 
-// Plan is the top-level representation of the json format of a plan. It includes
+	ResourceInstanceReplaceBecauseCannotUpdate    = "replace_because_cannot_update"
+	ResourceInstanceReplaceBecauseTainted         = "replace_because_tainted"
+	ResourceInstanceReplaceByRequest              = "replace_by_request"
+	ResourceInstanceReplaceByTriggers             = "replace_by_triggers"
+	ResourceInstanceDeleteBecauseNoResourceConfig = "delete_because_no_resource_config"
+	ResourceInstanceDeleteBecauseWrongRepetition  = "delete_because_wrong_repetition"
+	ResourceInstanceDeleteBecauseCountIndex       = "delete_because_count_index"
+	ResourceInstanceDeleteBecauseEachKey          = "delete_because_each_key"
+	ResourceInstanceDeleteBecauseNoModule         = "delete_because_no_module"
+	ResourceInstanceDeleteBecauseNoMoveTarget     = "delete_because_no_move_target"
+	ResourceInstanceReadBecauseConfigUnknown      = "read_because_config_unknown"
+	ResourceInstanceReadBecauseDependencyPending  = "read_because_dependency_pending"
+	ResourceInstanceReadBecauseCheckNested        = "read_because_check_nested"
+)
+
+// plan is the top-level representation of the json format of a plan. It includes
 // the complete config and current state.
 type plan struct {
 	FormatVersion    string      `json:"format_version,omitempty"`
@@ -33,11 +55,15 @@ type plan struct {
 	PlannedValues    stateValues `json:"planned_values,omitempty"`
 	// ResourceDrift and ResourceChanges are sorted in a user-friendly order
 	// that is undefined at this time, but consistent.
-	ResourceDrift   []resourceChange  `json:"resource_drift,omitempty"`
-	ResourceChanges []resourceChange  `json:"resource_changes,omitempty"`
-	OutputChanges   map[string]change `json:"output_changes,omitempty"`
-	PriorState      json.RawMessage   `json:"prior_state,omitempty"`
-	Config          json.RawMessage   `json:"configuration,omitempty"`
+	ResourceDrift      []ResourceChange  `json:"resource_drift,omitempty"`
+	ResourceChanges    []ResourceChange  `json:"resource_changes,omitempty"`
+	OutputChanges      map[string]Change `json:"output_changes,omitempty"`
+	PriorState         json.RawMessage   `json:"prior_state,omitempty"`
+	Config             json.RawMessage   `json:"configuration,omitempty"`
+	RelevantAttributes []ResourceAttr    `json:"relevant_attributes,omitempty"`
+	Checks             json.RawMessage   `json:"checks,omitempty"`
+	Timestamp          string            `json:"timestamp,omitempty"`
+	Errored            bool              `json:"errored"`
 }
 
 func newPlan() *plan {
@@ -46,8 +72,15 @@ func newPlan() *plan {
 	}
 }
 
+// ResourceAttr contains the address and attribute of an external for the
+// RelevantAttributes in the plan.
+type ResourceAttr struct {
+	Resource string          `json:"resource"`
+	Attr     json.RawMessage `json:"attribute"`
+}
+
 // Change is the representation of a proposed change for an object.
-type change struct {
+type Change struct {
 	// Actions are the actions that will be taken on the object selected by the
 	// properties below. Valid actions values are:
 	//    ["no-op"]
@@ -93,10 +126,33 @@ type change struct {
 	// consists of one or more steps, each of which will be a number or a
 	// string.
 	ReplacePaths json.RawMessage `json:"replace_paths,omitempty"`
+
+	// Importing contains the import metadata about this operation. If importing
+	// is present (ie. not null) then the change is an import operation in
+	// addition to anything mentioned in the actions field. The actual contents
+	// of the Importing struct is subject to change, so downstream consumers
+	// should treat any values in here as strictly optional.
+	Importing *Importing `json:"importing,omitempty"`
+
+	// GeneratedConfig contains any HCL config generated for this resource
+	// during planning as a string.
+	//
+	// If this is populated, then Importing should also be populated but this
+	// might change in the future. However, nNot all Importing changes will
+	// contain generated config.
+	GeneratedConfig string `json:"generated_config,omitempty"`
+}
+
+// Importing is a nested object for the resource import metadata.
+type Importing struct {
+	// The original ID of this resource used to target it as part of planned
+	// import operation.
+	ID string `json:"id,omitempty"`
 }
 
 type output struct {
 	Sensitive bool            `json:"sensitive"`
+	Type      json.RawMessage `json:"type,omitempty"`
 	Value     json.RawMessage `json:"value,omitempty"`
 }
 
@@ -108,6 +164,54 @@ type variable struct {
 	Value json.RawMessage `json:"value,omitempty"`
 }
 
+// MarshalForRenderer returns the pre-json encoding changes of the requested
+// plan, in a format available to the structured renderer.
+//
+// This function does a small part of the Marshal function, as it only returns
+// the part of the plan required by the jsonformat.Plan renderer.
+func MarshalForRenderer(
+	p *plans.Plan,
+	schemas *terraform.Schemas,
+) (map[string]Change, []ResourceChange, []ResourceChange, []ResourceAttr, error) {
+	output := newPlan()
+
+	var err error
+	if output.OutputChanges, err = MarshalOutputChanges(p.Changes); err != nil {
+		return nil, nil, nil, nil, err
+	}
+
+	if output.ResourceChanges, err = MarshalResourceChanges(p.Changes.Resources, schemas); err != nil {
+		return nil, nil, nil, nil, err
+	}
+
+	if len(p.DriftedResources) > 0 {
+		// In refresh-only mode, we render all resources marked as drifted,
+		// including those which have moved without other changes. In other plan
+		// modes, move-only changes will be included in the planned changes, so
+		// we skip them here.
+		var driftedResources []*plans.ResourceInstanceChangeSrc
+		if p.UIMode == plans.RefreshOnlyMode {
+			driftedResources = p.DriftedResources
+		} else {
+			for _, dr := range p.DriftedResources {
+				if dr.Action != plans.NoOp {
+					driftedResources = append(driftedResources, dr)
+				}
+			}
+		}
+		output.ResourceDrift, err = MarshalResourceChanges(driftedResources, schemas)
+		if err != nil {
+			return nil, nil, nil, nil, err
+		}
+	}
+
+	if err := output.marshalRelevantAttrs(p); err != nil {
+		return nil, nil, nil, nil, err
+	}
+
+	return output.OutputChanges, output.ResourceChanges, output.ResourceDrift, output.RelevantAttributes, nil
+}
+
 // Marshal returns the json encoding of a terraform plan.
 func Marshal(
 	config *configs.Config,
@@ -117,8 +221,10 @@ func Marshal(
 ) ([]byte, error) {
 	output := newPlan()
 	output.TerraformVersion = version.String()
+	output.Timestamp = p.Timestamp.Format(time.RFC3339)
+	output.Errored = p.Errored
 
-	err := output.marshalPlanVariables(p.VariableValues, schemas)
+	err := output.marshalPlanVariables(p.VariableValues, config.Module.Variables)
 	if err != nil {
 		return nil, fmt.Errorf("error in marshalPlanVariables: %s", err)
 	}
@@ -145,26 +251,34 @@ func Marshal(
 				}
 			}
 		}
-		output.ResourceDrift, err = output.marshalResourceChanges(driftedResources, schemas)
+		output.ResourceDrift, err = MarshalResourceChanges(driftedResources, schemas)
 		if err != nil {
 			return nil, fmt.Errorf("error in marshaling resource drift: %s", err)
 		}
 	}
 
+	if err := output.marshalRelevantAttrs(p); err != nil {
+		return nil, fmt.Errorf("error marshaling relevant attributes for external changes: %s", err)
+	}
+
 	// output.ResourceChanges
 	if p.Changes != nil {
-		output.ResourceChanges, err = output.marshalResourceChanges(p.Changes.Resources, schemas)
+		output.ResourceChanges, err = MarshalResourceChanges(p.Changes.Resources, schemas)
 		if err != nil {
 			return nil, fmt.Errorf("error in marshaling resource changes: %s", err)
 		}
 	}
 
 	// output.OutputChanges
-	err = output.marshalOutputChanges(p.Changes)
-	if err != nil {
+	if output.OutputChanges, err = MarshalOutputChanges(p.Changes); err != nil {
 		return nil, fmt.Errorf("error in marshaling output changes: %s", err)
 	}
 
+	// output.Checks
+	if p.Checks != nil && p.Checks.ConfigResults.Len() > 0 {
+		output.Checks = jsonchecks.MarshalCheckStates(p.Checks)
+	}
+
 	// output.PriorState
 	if sf != nil && !sf.State.Empty() {
 		output.PriorState, err = jsonstate.Marshal(sf, schemas)
@@ -179,15 +293,10 @@ func Marshal(
 		return nil, fmt.Errorf("error marshaling config: %s", err)
 	}
 
-	ret, err := json.Marshal(output)
-	return ret, err
+	return json.Marshal(output)
 }
 
-func (p *plan) marshalPlanVariables(vars map[string]plans.DynamicValue, schemas *terraform.Schemas) error {
-	if len(vars) == 0 {
-		return nil
-	}
-
+func (p *plan) marshalPlanVariables(vars map[string]plans.DynamicValue, decls map[string]*configs.Variable) error {
 	p.Variables = make(variables, len(vars))
 
 	for k, v := range vars {
@@ -203,14 +312,64 @@ func (p *plan) marshalPlanVariables(vars map[string]plans.DynamicValue, schemas
 			Value: valJSON,
 		}
 	}
+
+	// In Terraform v1.1 and earlier we had some confusion about which subsystem
+	// of Terraform was the one responsible for substituting in default values
+	// for unset module variables, with root module variables being handled in
+	// three different places while child module variables were only handled
+	// during the Terraform Core graph walk.
+	//
+	// For Terraform v1.2 and later we rationalized that by having the Terraform
+	// Core graph walk always be responsible for selecting defaults regardless
+	// of root vs. child module, but unfortunately our earlier accidental
+	// misbehavior bled out into the public interface by making the defaults
+	// show up in the "vars" map to this function. Those are now correctly
+	// omitted (so that the plan file only records the variables _actually_
+	// set by the caller) but consumers of the JSON plan format may be depending
+	// on our old behavior and so we'll fake it here just in time so that
+	// outside consumers won't see a behavior change.
+	for name, decl := range decls {
+		if _, ok := p.Variables[name]; ok {
+			continue
+		}
+		if val := decl.Default; val != cty.NilVal {
+			valJSON, err := ctyjson.Marshal(val, val.Type())
+			if err != nil {
+				return err
+			}
+			p.Variables[name] = &variable{
+				Value: valJSON,
+			}
+		}
+	}
+
+	if len(p.Variables) == 0 {
+		p.Variables = nil // omit this property if there are no variables to describe
+	}
+
 	return nil
 }
 
-func (p *plan) marshalResourceChanges(resources []*plans.ResourceInstanceChangeSrc, schemas *terraform.Schemas) ([]resourceChange, error) {
-	var ret []resourceChange
+// MarshalResourceChanges converts the provided internal representation of
+// ResourceInstanceChangeSrc objects into the public structured JSON changes.
+//
+// This function is referenced directly from the structured renderer tests, to
+// ensure parity between the renderers. It probably shouldn't be used anywhere
+// else.
+func MarshalResourceChanges(resources []*plans.ResourceInstanceChangeSrc, schemas *terraform.Schemas) ([]ResourceChange, error) {
+	var ret []ResourceChange
 
-	for _, rc := range resources {
-		var r resourceChange
+	var sortedResources []*plans.ResourceInstanceChangeSrc
+	sortedResources = append(sortedResources, resources...)
+	sort.Slice(sortedResources, func(i, j int) bool {
+		if !sortedResources[i].Addr.Equal(sortedResources[j].Addr) {
+			return sortedResources[i].Addr.Less(sortedResources[j].Addr)
+		}
+		return sortedResources[i].DeposedKey < sortedResources[j].DeposedKey
+	})
+
+	for _, rc := range sortedResources {
+		var r ResourceChange
 		addr := rc.Addr
 		r.Address = addr.String()
 		if !addr.Equal(rc.PrevRunAddr) {
@@ -301,7 +460,12 @@ func (p *plan) marshalResourceChanges(resources []*plans.ResourceInstanceChangeS
 			return nil, err
 		}
 
-		r.Change = change{
+		var importing *Importing
+		if rc.Importing != nil {
+			importing = &Importing{ID: rc.Importing.ID}
+		}
+
+		r.Change = Change{
 			Actions:         actionString(rc.Action.String()),
 			Before:          json.RawMessage(before),
 			After:           json.RawMessage(after),
@@ -309,6 +473,8 @@ func (p *plan) marshalResourceChanges(resources []*plans.ResourceInstanceChangeS
 			BeforeSensitive: json.RawMessage(beforeSensitive),
 			AfterSensitive:  json.RawMessage(afterSensitive),
 			ReplacePaths:    replacePaths,
+			Importing:       importing,
+			GeneratedConfig: rc.GeneratedConfig,
 		}
 
 		if rc.DeposedKey != states.NotDeposed {
@@ -317,14 +483,17 @@ func (p *plan) marshalResourceChanges(resources []*plans.ResourceInstanceChangeS
 
 		key := addr.Resource.Key
 		if key != nil {
-			r.Index = key
+			value := key.Value()
+			if r.Index, err = ctyjson.Marshal(value, value.Type()); err != nil {
+				return nil, err
+			}
 		}
 
 		switch addr.Resource.Resource.Mode {
 		case addrs.ManagedResourceMode:
-			r.Mode = "managed"
+			r.Mode = jsonstate.ManagedResourceMode
 		case addrs.DataResourceMode:
-			r.Mode = "data"
+			r.Mode = jsonstate.DataResourceMode
 		default:
 			return nil, fmt.Errorf("resource %s has an unsupported mode %s", r.Address, addr.Resource.Resource.Mode.String())
 		}
@@ -337,21 +506,31 @@ func (p *plan) marshalResourceChanges(resources []*plans.ResourceInstanceChangeS
 		case plans.ResourceInstanceChangeNoReason:
 			r.ActionReason = "" // will be omitted in output
 		case plans.ResourceInstanceReplaceBecauseCannotUpdate:
-			r.ActionReason = "replace_because_cannot_update"
+			r.ActionReason = ResourceInstanceReplaceBecauseCannotUpdate
 		case plans.ResourceInstanceReplaceBecauseTainted:
-			r.ActionReason = "replace_because_tainted"
+			r.ActionReason = ResourceInstanceReplaceBecauseTainted
 		case plans.ResourceInstanceReplaceByRequest:
-			r.ActionReason = "replace_by_request"
+			r.ActionReason = ResourceInstanceReplaceByRequest
+		case plans.ResourceInstanceReplaceByTriggers:
+			r.ActionReason = ResourceInstanceReplaceByTriggers
 		case plans.ResourceInstanceDeleteBecauseNoResourceConfig:
-			r.ActionReason = "delete_because_no_resource_config"
+			r.ActionReason = ResourceInstanceDeleteBecauseNoResourceConfig
 		case plans.ResourceInstanceDeleteBecauseWrongRepetition:
-			r.ActionReason = "delete_because_wrong_repetition"
+			r.ActionReason = ResourceInstanceDeleteBecauseWrongRepetition
 		case plans.ResourceInstanceDeleteBecauseCountIndex:
-			r.ActionReason = "delete_because_count_index"
+			r.ActionReason = ResourceInstanceDeleteBecauseCountIndex
 		case plans.ResourceInstanceDeleteBecauseEachKey:
-			r.ActionReason = "delete_because_each_key"
+			r.ActionReason = ResourceInstanceDeleteBecauseEachKey
 		case plans.ResourceInstanceDeleteBecauseNoModule:
-			r.ActionReason = "delete_because_no_module"
+			r.ActionReason = ResourceInstanceDeleteBecauseNoModule
+		case plans.ResourceInstanceDeleteBecauseNoMoveTarget:
+			r.ActionReason = ResourceInstanceDeleteBecauseNoMoveTarget
+		case plans.ResourceInstanceReadBecauseConfigUnknown:
+			r.ActionReason = ResourceInstanceReadBecauseConfigUnknown
+		case plans.ResourceInstanceReadBecauseDependencyPending:
+			r.ActionReason = ResourceInstanceReadBecauseDependencyPending
+		case plans.ResourceInstanceReadBecauseCheckNested:
+			r.ActionReason = ResourceInstanceReadBecauseCheckNested
 		default:
 			return nil, fmt.Errorf("resource %s has an unsupported action reason %s", r.Address, rc.ActionReason)
 		}
@@ -360,24 +539,35 @@ func (p *plan) marshalResourceChanges(resources []*plans.ResourceInstanceChangeS
 
 	}
 
-	sort.Slice(ret, func(i, j int) bool {
-		return ret[i].Address < ret[j].Address
-	})
-
 	return ret, nil
 }
 
-func (p *plan) marshalOutputChanges(changes *plans.Changes) error {
+// MarshalOutputChanges converts the provided internal representation of
+// Changes objects into the structured JSON representation.
+//
+// This function is referenced directly from the structured renderer tests, to
+// ensure parity between the renderers. It probably shouldn't be used anywhere
+// else.
+func MarshalOutputChanges(changes *plans.Changes) (map[string]Change, error) {
 	if changes == nil {
 		// Nothing to do!
-		return nil
+		return nil, nil
 	}
 
-	p.OutputChanges = make(map[string]change, len(changes.Outputs))
+	outputChanges := make(map[string]Change, len(changes.Outputs))
 	for _, oc := range changes.Outputs {
+
+		// Skip output changes that are not from the root module.
+		// These are automatically stripped from plans that are written to disk
+		// elsewhere, we just need to duplicate the logic here in case anyone
+		// is converting this plan directly from memory.
+		if !oc.Addr.Module.IsRoot() {
+			continue
+		}
+
 		changeV, err := oc.Decode()
 		if err != nil {
-			return err
+			return nil, err
 		}
 		// We drop the marks from the change, as decoding is only an
 		// intermediate step to re-encode the values as json
@@ -385,21 +575,32 @@ func (p *plan) marshalOutputChanges(changes *plans.Changes) error {
 		changeV.After, _ = changeV.After.UnmarkDeep()
 
 		var before, after []byte
-		afterUnknown := cty.False
+		var afterUnknown cty.Value
+
 		if changeV.Before != cty.NilVal {
 			before, err = ctyjson.Marshal(changeV.Before, changeV.Before.Type())
 			if err != nil {
-				return err
+				return nil, err
 			}
 		}
 		if changeV.After != cty.NilVal {
 			if changeV.After.IsWhollyKnown() {
 				after, err = ctyjson.Marshal(changeV.After, changeV.After.Type())
 				if err != nil {
-					return err
+					return nil, err
 				}
+				afterUnknown = cty.False
 			} else {
-				afterUnknown = cty.True
+				filteredAfter := omitUnknowns(changeV.After)
+				if filteredAfter.IsNull() {
+					after = nil
+				} else {
+					after, err = ctyjson.Marshal(filteredAfter, filteredAfter.Type())
+					if err != nil {
+						return nil, err
+					}
+				}
+				afterUnknown = unknownAsBool(changeV.After)
 			}
 		}
 
@@ -413,24 +614,28 @@ func (p *plan) marshalOutputChanges(changes *plans.Changes) error {
 		}
 		sensitive, err := ctyjson.Marshal(outputSensitive, outputSensitive.Type())
 		if err != nil {
-			return err
+			return nil, err
 		}
 
 		a, _ := ctyjson.Marshal(afterUnknown, afterUnknown.Type())
 
-		c := change{
+		c := Change{
 			Actions:         actionString(oc.Action.String()),
 			Before:          json.RawMessage(before),
 			After:           json.RawMessage(after),
 			AfterUnknown:    a,
 			BeforeSensitive: json.RawMessage(sensitive),
 			AfterSensitive:  json.RawMessage(sensitive),
+
+			// Just to be explicit, outputs cannot be imported so this is always
+			// nil.
+			Importing: nil,
 		}
 
-		p.OutputChanges[oc.Addr.OutputValue.Name] = c
+		outputChanges[oc.Addr.OutputValue.Name] = c
 	}
 
-	return nil
+	return outputChanges, nil
 }
 
 func (p *plan) marshalPlannedValues(changes *plans.Changes, schemas *terraform.Schemas) error {
@@ -451,6 +656,19 @@ func (p *plan) marshalPlannedValues(changes *plans.Changes, schemas *terraform.S
 	return nil
 }
 
+func (p *plan) marshalRelevantAttrs(plan *plans.Plan) error {
+	for _, ra := range plan.RelevantAttributes {
+		addr := ra.Resource.String()
+		path, err := encodePath(ra.Attr)
+		if err != nil {
+			return err
+		}
+
+		p.RelevantAttributes = append(p.RelevantAttributes, ResourceAttr{addr, path})
+	}
+	return nil
+}
+
 // omitUnknowns recursively walks the src cty.Value and returns a new cty.Value,
 // omitting any unknowns.
 //
@@ -475,8 +693,9 @@ func omitUnknowns(val cty.Value) cty.Value {
 			newVal := omitUnknowns(v)
 			if newVal != cty.NilVal {
 				vals = append(vals, newVal)
-			} else if newVal == cty.NilVal && ty.IsListType() {
-				// list length may be significant, so we will turn unknowns into nulls
+			} else if newVal == cty.NilVal {
+				// element order is how we correlate unknownness, so we must
+				// replace unknowns with nulls
 				vals = append(vals, cty.NullVal(v.Type()))
 			}
 		}
@@ -602,10 +821,40 @@ func actionString(action string) []string {
 	}
 }
 
+// UnmarshalActions reverses the actionString function.
+func UnmarshalActions(actions []string) plans.Action {
+	if len(actions) == 2 {
+		if actions[0] == "create" && actions[1] == "delete" {
+			return plans.CreateThenDelete
+		}
+
+		if actions[0] == "delete" && actions[1] == "create" {
+			return plans.DeleteThenCreate
+		}
+	}
+
+	if len(actions) == 1 {
+		switch actions[0] {
+		case "create":
+			return plans.Create
+		case "delete":
+			return plans.Delete
+		case "update":
+			return plans.Update
+		case "read":
+			return plans.Read
+		case "no-op":
+			return plans.NoOp
+		}
+	}
+
+	panic("unrecognized action slice: " + strings.Join(actions, ", "))
+}
+
 // encodePaths lossily encodes a cty.PathSet into an array of arrays of step
 // values, such as:
 //
-//   [["length"],["triggers",0,"value"]]
+//	[["length"],["triggers",0,"value"]]
 //
 // The lossiness is that we cannot distinguish between an IndexStep with string
 // key and a GetAttr step. This is fine with JSON output, because JSON's type
@@ -613,8 +862,8 @@ func actionString(action string) []string {
 // indexes.
 //
 // JavaScript (or similar dynamic language) consumers of these values can
-// recursively apply the steps to a given object using an index operation for
-// each step.
+// iterate over the the steps starting from the root object to reach the
+// value that each path is describing.
 func encodePaths(pathSet cty.PathSet) (json.RawMessage, error) {
 	if pathSet.Empty() {
 		return nil, nil
@@ -624,26 +873,7 @@ func encodePaths(pathSet cty.PathSet) (json.RawMessage, error) {
 	jsonPaths := make([]json.RawMessage, 0, len(pathList))
 
 	for _, path := range pathList {
-		steps := make([]json.RawMessage, 0, len(path))
-		for _, step := range path {
-			switch s := step.(type) {
-			case cty.IndexStep:
-				key, err := ctyjson.Marshal(s.Key, s.Key.Type())
-				if err != nil {
-					return nil, fmt.Errorf("Failed to marshal index step key %#v: %s", s.Key, err)
-				}
-				steps = append(steps, key)
-			case cty.GetAttrStep:
-				name, err := json.Marshal(s.Name)
-				if err != nil {
-					return nil, fmt.Errorf("Failed to marshal get attr step name %#v: %s", s.Name, err)
-				}
-				steps = append(steps, name)
-			default:
-				return nil, fmt.Errorf("Unsupported path step %#v (%t)", step, step)
-			}
-		}
-		jsonPath, err := json.Marshal(steps)
+		jsonPath, err := encodePath(path)
 		if err != nil {
 			return nil, err
 		}
@@ -652,3 +882,26 @@ func encodePaths(pathSet cty.PathSet) (json.RawMessage, error) {
 
 	return json.Marshal(jsonPaths)
 }
+
+func encodePath(path cty.Path) (json.RawMessage, error) {
+	steps := make([]json.RawMessage, 0, len(path))
+	for _, step := range path {
+		switch s := step.(type) {
+		case cty.IndexStep:
+			key, err := ctyjson.Marshal(s.Key, s.Key.Type())
+			if err != nil {
+				return nil, fmt.Errorf("Failed to marshal index step key %#v: %s", s.Key, err)
+			}
+			steps = append(steps, key)
+		case cty.GetAttrStep:
+			name, err := json.Marshal(s.Name)
+			if err != nil {
+				return nil, fmt.Errorf("Failed to marshal get attr step name %#v: %s", s.Name, err)
+			}
+			steps = append(steps, name)
+		default:
+			return nil, fmt.Errorf("Unsupported path step %#v (%t)", step, step)
+		}
+	}
+	return json.Marshal(steps)
+}
