@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package jsonplan
 
 import (
@@ -23,7 +26,7 @@ type stateValues struct {
 	RootModule module            `json:"root_module,omitempty"`
 }
 
-// attributeValues is the JSON representation of the attribute values of the
+// AttributeValues is the JSON representation of the attribute values of the
 // resource, whose structure depends on the resource type schema.
 type attributeValues map[string]interface{}
 
@@ -57,7 +60,7 @@ func marshalPlannedOutputs(changes *plans.Changes) (map[string]output, error) {
 			continue
 		}
 
-		var after []byte
+		var after, afterType []byte
 		changeV, err := oc.Decode()
 		if err != nil {
 			return ret, err
@@ -68,7 +71,12 @@ func marshalPlannedOutputs(changes *plans.Changes) (map[string]output, error) {
 		changeV.After, _ = changeV.After.UnmarkDeep()
 
 		if changeV.After != cty.NilVal && changeV.After.IsWhollyKnown() {
-			after, err = ctyjson.Marshal(changeV.After, changeV.After.Type())
+			ty := changeV.After.Type()
+			after, err = ctyjson.Marshal(changeV.After, ty)
+			if err != nil {
+				return ret, err
+			}
+			afterType, err = ctyjson.MarshalType(ty)
 			if err != nil {
 				return ret, err
 			}
@@ -76,6 +84,7 @@ func marshalPlannedOutputs(changes *plans.Changes) (map[string]output, error) {
 
 		ret[oc.Addr.OutputValue.Name] = output{
 			Value:     json.RawMessage(after),
+			Type:      json.RawMessage(afterType),
 			Sensitive: oc.Sensitive,
 		}
 	}
