@@ -1,6 +1,10 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package command
 
 import (
+	"context"
 	"io/ioutil"
 	"os"
 	"path/filepath"
@@ -28,9 +32,8 @@ import (
 // Test empty directory with no config/state creates a local state.
 func TestMetaBackend_emptyDir(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	os.MkdirAll(td, 0755)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Get the backend
@@ -46,7 +49,7 @@ func TestMetaBackend_emptyDir(t *testing.T) {
 		t.Fatalf("unexpected error: %s", err)
 	}
 	s.WriteState(testState())
-	if err := s.PersistState(); err != nil {
+	if err := s.PersistState(nil); err != nil {
 		t.Fatalf("unexpected error: %s", err)
 	}
 
@@ -84,9 +87,8 @@ func isEmptyState(path string) bool {
 // use the legacy state.
 func TestMetaBackend_emptyWithDefaultState(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	os.MkdirAll(td, 0755)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Write the legacy state
@@ -136,7 +138,7 @@ func TestMetaBackend_emptyWithDefaultState(t *testing.T) {
 	next := testState()
 	next.RootModule().SetOutputValue("foo", cty.StringVal("bar"), false)
 	s.WriteState(next)
-	if err := s.PersistState(); err != nil {
+	if err := s.PersistState(nil); err != nil {
 		t.Fatalf("unexpected error: %s", err)
 	}
 
@@ -149,15 +151,13 @@ func TestMetaBackend_emptyWithDefaultState(t *testing.T) {
 // Test an empty directory with an explicit state path (outside the dir)
 func TestMetaBackend_emptyWithExplicitState(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	os.MkdirAll(td, 0755)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Create another directory to store our state
-	stateDir := tempDir(t)
+	stateDir := t.TempDir()
 	os.MkdirAll(stateDir, 0755)
-	defer os.RemoveAll(stateDir)
 
 	// Write the legacy state
 	statePath := filepath.Join(stateDir, "foo")
@@ -209,7 +209,7 @@ func TestMetaBackend_emptyWithExplicitState(t *testing.T) {
 	next := testState()
 	markStateForMatching(next, "bar") // just any change so it shows as different than before
 	s.WriteState(next)
-	if err := s.PersistState(); err != nil {
+	if err := s.PersistState(nil); err != nil {
 		t.Fatalf("unexpected error: %s", err)
 	}
 
@@ -222,9 +222,8 @@ func TestMetaBackend_emptyWithExplicitState(t *testing.T) {
 // Verify that interpolations result in an error
 func TestMetaBackend_configureInterpolation(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-new-interp"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Setup the meta
@@ -239,9 +238,8 @@ func TestMetaBackend_configureInterpolation(t *testing.T) {
 
 // Newly configured backend
 func TestMetaBackend_configureNew(t *testing.T) {
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-new"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Setup the meta
@@ -271,7 +269,7 @@ func TestMetaBackend_configureNew(t *testing.T) {
 	mark := markStateForMatching(state, "changing")
 
 	s.WriteState(state)
-	if err := s.PersistState(); err != nil {
+	if err := s.PersistState(nil); err != nil {
 		t.Fatalf("unexpected error: %s", err)
 	}
 
@@ -304,9 +302,8 @@ func TestMetaBackend_configureNew(t *testing.T) {
 // Newly configured backend with prior local state and no remote state
 func TestMetaBackend_configureNewWithState(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-new-migrate"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Ask input
@@ -346,7 +343,7 @@ func TestMetaBackend_configureNewWithState(t *testing.T) {
 	state = states.NewState()
 	mark := markStateForMatching(state, "changing")
 
-	if err := statemgr.WriteAndPersist(s, state); err != nil {
+	if err := statemgr.WriteAndPersist(s, state, nil); err != nil {
 		t.Fatalf("unexpected error: %s", err)
 	}
 
@@ -382,9 +379,8 @@ func TestMetaBackend_configureNewWithState(t *testing.T) {
 // for copy.
 func TestMetaBackend_configureNewWithoutCopy(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-new-migrate"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	if err := copy.CopyFile(DefaultStateFilename, "local-state.tfstate"); err != nil {
@@ -433,9 +429,8 @@ func TestMetaBackend_configureNewWithoutCopy(t *testing.T) {
 // but opting to not migrate.
 func TestMetaBackend_configureNewWithStateNoMigrate(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-new-migrate"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Ask input
@@ -478,9 +473,8 @@ func TestMetaBackend_configureNewWithStateNoMigrate(t *testing.T) {
 // Newly configured backend with prior local state and remote state
 func TestMetaBackend_configureNewWithStateExisting(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-new-migrate-existing"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Setup the meta
@@ -515,7 +509,7 @@ func TestMetaBackend_configureNewWithStateExisting(t *testing.T) {
 	mark := markStateForMatching(state, "changing")
 
 	s.WriteState(state)
-	if err := s.PersistState(); err != nil {
+	if err := s.PersistState(nil); err != nil {
 		t.Fatalf("unexpected error: %s", err)
 	}
 
@@ -550,9 +544,8 @@ func TestMetaBackend_configureNewWithStateExisting(t *testing.T) {
 // Newly configured backend with prior local state and remote state
 func TestMetaBackend_configureNewWithStateExistingNoMigrate(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-new-migrate-existing"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Ask input
@@ -587,7 +580,7 @@ func TestMetaBackend_configureNewWithStateExistingNoMigrate(t *testing.T) {
 	state = states.NewState()
 	mark := markStateForMatching(state, "changing")
 	s.WriteState(state)
-	if err := s.PersistState(); err != nil {
+	if err := s.PersistState(nil); err != nil {
 		t.Fatalf("unexpected error: %s", err)
 	}
 
@@ -662,9 +655,8 @@ func TestMetaBackend_configuredUnchanged(t *testing.T) {
 // Changing a configured backend
 func TestMetaBackend_configuredChange(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-change"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Ask input
@@ -707,7 +699,7 @@ func TestMetaBackend_configuredChange(t *testing.T) {
 	mark := markStateForMatching(state, "changing")
 
 	s.WriteState(state)
-	if err := s.PersistState(); err != nil {
+	if err := s.PersistState(nil); err != nil {
 		t.Fatalf("unexpected error: %s", err)
 	}
 
@@ -742,9 +734,8 @@ func TestMetaBackend_configuredChange(t *testing.T) {
 // backend is if this is the first time.
 func TestMetaBackend_reconfigureChange(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-change-single-to-single"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Register the single-state backend
@@ -796,9 +787,8 @@ func TestMetaBackend_reconfigureChange(t *testing.T) {
 // is available.
 func TestMetaBackend_initSelectedWorkspaceDoesNotExist(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("init-backend-selected-workspace-doesnt-exist-multi"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Setup the meta
@@ -830,9 +820,8 @@ func TestMetaBackend_initSelectedWorkspaceDoesNotExist(t *testing.T) {
 // automatically select that single workspace.
 func TestMetaBackend_initSelectedWorkspaceDoesNotExistAutoSelect(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("init-backend-selected-workspace-doesnt-exist-single"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Setup the meta
@@ -872,9 +861,8 @@ func TestMetaBackend_initSelectedWorkspaceDoesNotExistAutoSelect(t *testing.T) {
 // the currently selected workspace with input=false should fail.
 func TestMetaBackend_initSelectedWorkspaceDoesNotExistInputFalse(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("init-backend-selected-workspace-doesnt-exist-multi"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Setup the meta
@@ -893,9 +881,8 @@ func TestMetaBackend_initSelectedWorkspaceDoesNotExistInputFalse(t *testing.T) {
 // Changing a configured backend, copying state
 func TestMetaBackend_configuredChangeCopy(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-change"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Ask input
@@ -941,9 +928,8 @@ func TestMetaBackend_configuredChangeCopy(t *testing.T) {
 // backend that only supports single states.
 func TestMetaBackend_configuredChangeCopy_singleState(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-change-single-to-single"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Register the single-state backend
@@ -996,9 +982,8 @@ func TestMetaBackend_configuredChangeCopy_singleState(t *testing.T) {
 // a default state.
 func TestMetaBackend_configuredChangeCopy_multiToSingleDefault(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-change-multi-default-to-single"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Register the single-state backend
@@ -1050,9 +1035,8 @@ func TestMetaBackend_configuredChangeCopy_multiToSingleDefault(t *testing.T) {
 // backend that only supports single states.
 func TestMetaBackend_configuredChangeCopy_multiToSingle(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-change-multi-to-single"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Register the single-state backend
@@ -1120,9 +1104,8 @@ func TestMetaBackend_configuredChangeCopy_multiToSingle(t *testing.T) {
 // backend that only supports single states.
 func TestMetaBackend_configuredChangeCopy_multiToSingleCurrentEnv(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-change-multi-to-single"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Register the single-state backend
@@ -1186,9 +1169,8 @@ func TestMetaBackend_configuredChangeCopy_multiToSingleCurrentEnv(t *testing.T)
 // backend that also supports multi-state.
 func TestMetaBackend_configuredChangeCopy_multiToMulti(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-change-multi-to-multi"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Ask input
@@ -1280,9 +1262,8 @@ func TestMetaBackend_configuredChangeCopy_multiToMulti(t *testing.T) {
 // default state while the default state is non-empty.
 func TestMetaBackend_configuredChangeCopy_multiToNoDefaultWithDefault(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-change-multi-to-no-default-with-default"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Register the single-state backend
@@ -1356,9 +1337,8 @@ func TestMetaBackend_configuredChangeCopy_multiToNoDefaultWithDefault(t *testing
 // default state while the default state is empty.
 func TestMetaBackend_configuredChangeCopy_multiToNoDefaultWithoutDefault(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-change-multi-to-no-default-without-default"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Register the single-state backend
@@ -1429,9 +1409,8 @@ func TestMetaBackend_configuredChangeCopy_multiToNoDefaultWithoutDefault(t *test
 // Unsetting a saved backend
 func TestMetaBackend_configuredUnset(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-unset"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Ask input
@@ -1473,7 +1452,7 @@ func TestMetaBackend_configuredUnset(t *testing.T) {
 
 	// Write some state
 	s.WriteState(testState())
-	if err := s.PersistState(); err != nil {
+	if err := s.PersistState(nil); err != nil {
 		t.Fatalf("unexpected error: %s", err)
 	}
 
@@ -1492,9 +1471,8 @@ func TestMetaBackend_configuredUnset(t *testing.T) {
 // Unsetting a saved backend and copying the remote state
 func TestMetaBackend_configuredUnsetCopy(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-unset"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// Ask input
@@ -1532,7 +1510,7 @@ func TestMetaBackend_configuredUnsetCopy(t *testing.T) {
 
 	// Write some state
 	s.WriteState(testState())
-	if err := s.PersistState(); err != nil {
+	if err := s.PersistState(nil); err != nil {
 		t.Fatalf("unexpected error: %s", err)
 	}
 
@@ -1550,9 +1528,8 @@ func TestMetaBackend_configuredUnsetCopy(t *testing.T) {
 // A plan that has uses the local backend
 func TestMetaBackend_planLocal(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-plan-local"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	backendConfigBlock := cty.ObjectVal(map[string]cty.Value{
@@ -1573,7 +1550,7 @@ func TestMetaBackend_planLocal(t *testing.T) {
 	m := testMetaBackend(t, nil)
 
 	// Get the backend
-	b, diags := m.BackendForPlan(backendConfig)
+	b, diags := m.BackendForLocalPlan(backendConfig)
 	if diags.HasErrors() {
 		t.Fatal(diags.Err())
 	}
@@ -1612,7 +1589,7 @@ func TestMetaBackend_planLocal(t *testing.T) {
 	mark := markStateForMatching(state, "changing")
 
 	s.WriteState(state)
-	if err := s.PersistState(); err != nil {
+	if err := s.PersistState(nil); err != nil {
 		t.Fatalf("unexpected error: %s", err)
 	}
 
@@ -1639,9 +1616,8 @@ func TestMetaBackend_planLocal(t *testing.T) {
 
 // A plan with a custom state save path
 func TestMetaBackend_planLocalStatePath(t *testing.T) {
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-plan-local"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	original := testState()
@@ -1675,7 +1651,7 @@ func TestMetaBackend_planLocalStatePath(t *testing.T) {
 	m.stateOutPath = statePath
 
 	// Get the backend
-	b, diags := m.BackendForPlan(plannedBackend)
+	b, diags := m.BackendForLocalPlan(plannedBackend)
 	if diags.HasErrors() {
 		t.Fatal(diags.Err())
 	}
@@ -1714,7 +1690,7 @@ func TestMetaBackend_planLocalStatePath(t *testing.T) {
 	mark := markStateForMatching(state, "changing")
 
 	s.WriteState(state)
-	if err := s.PersistState(); err != nil {
+	if err := s.PersistState(nil); err != nil {
 		t.Fatalf("unexpected error: %s", err)
 	}
 
@@ -1742,9 +1718,8 @@ func TestMetaBackend_planLocalStatePath(t *testing.T) {
 // A plan that has no backend config, matching local state
 func TestMetaBackend_planLocalMatch(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("backend-plan-local-match"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	backendConfigBlock := cty.ObjectVal(map[string]cty.Value{
@@ -1765,7 +1740,7 @@ func TestMetaBackend_planLocalMatch(t *testing.T) {
 	m := testMetaBackend(t, nil)
 
 	// Get the backend
-	b, diags := m.BackendForPlan(backendConfig)
+	b, diags := m.BackendForLocalPlan(backendConfig)
 	if diags.HasErrors() {
 		t.Fatal(diags.Err())
 	}
@@ -1802,7 +1777,7 @@ func TestMetaBackend_planLocalMatch(t *testing.T) {
 	mark := markStateForMatching(state, "changing")
 
 	s.WriteState(state)
-	if err := s.PersistState(); err != nil {
+	if err := s.PersistState(nil); err != nil {
 		t.Fatalf("unexpected error: %s", err)
 	}
 
@@ -1830,9 +1805,8 @@ func TestMetaBackend_planLocalMatch(t *testing.T) {
 // init a backend using -backend-config options multiple times
 func TestMetaBackend_configureWithExtra(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("init-backend-empty"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	extras := map[string]cty.Value{"path": cty.StringVal("hello")}
@@ -1882,9 +1856,8 @@ func TestMetaBackend_configureWithExtra(t *testing.T) {
 // when configuring a default local state, don't delete local state
 func TestMetaBackend_localDoesNotDeleteLocal(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("init-backend-empty"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// // create our local state
@@ -1910,9 +1883,8 @@ func TestMetaBackend_localDoesNotDeleteLocal(t *testing.T) {
 // move options from config to -backend-config
 func TestMetaBackend_configToExtra(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("init-backend"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// init the backend
@@ -1968,7 +1940,7 @@ func TestBackendFromState(t *testing.T) {
 	// them to match just for this test.
 	wd.OverrideDataDir(".")
 
-	stateBackend, diags := m.backendFromState()
+	stateBackend, diags := m.backendFromState(context.Background())
 	if diags.HasErrors() {
 		t.Fatal(diags.Err())
 	}
