@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package command
 
 import (
@@ -15,7 +18,6 @@ import (
 	terraformProvider "github.com/hashicorp/terraform/internal/builtin/providers/terraform"
 	"github.com/hashicorp/terraform/internal/getproviders"
 	"github.com/hashicorp/terraform/internal/logging"
-	"github.com/hashicorp/terraform/internal/moduletest"
 	tfplugin "github.com/hashicorp/terraform/internal/plugin"
 	tfplugin6 "github.com/hashicorp/terraform/internal/plugin6"
 	"github.com/hashicorp/terraform/internal/providercache"
@@ -63,6 +65,7 @@ func (m *Meta) providerInstallerCustomSource(source getproviders.Source) *provid
 	inst := providercache.NewInstaller(targetDir, source)
 	if globalCacheDir != nil {
 		inst.SetGlobalCacheDir(globalCacheDir)
+		inst.SetGlobalCacheDirMayBreakDependencyLockFile(m.PluginCacheMayBreakDependencyLockFile)
 	}
 	var builtinProviderTypes []string
 	for ty := range m.internalProviders() {
@@ -337,9 +340,6 @@ func (m *Meta) internalProviders() map[string]providers.Factory {
 		"terraform": func() (providers.Interface, error) {
 			return terraformProvider.NewProvider(), nil
 		},
-		"test": func() (providers.Interface, error) {
-			return moduletest.NewProvider(), nil
-		},
 	}
 }
 
@@ -382,10 +382,12 @@ func providerFactory(meta *providercache.CachedProvider) providers.Factory {
 		case 5:
 			p := raw.(*tfplugin.GRPCProvider)
 			p.PluginClient = client
+			p.Addr = meta.Provider
 			return p, nil
 		case 6:
 			p := raw.(*tfplugin6.GRPCProvider)
 			p.PluginClient = client
+			p.Addr = meta.Provider
 			return p, nil
 		default:
 			panic("unsupported protocol version")
