@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package json
 
 import (
@@ -8,10 +11,24 @@ import (
 
 func NewResourceInstanceChange(change *plans.ResourceInstanceChangeSrc) *ResourceInstanceChange {
 	c := &ResourceInstanceChange{
-		Resource: newResourceAddr(change.Addr),
-		Action:   changeAction(change.Action),
-		Reason:   changeReason(change.ActionReason),
+		Resource:        newResourceAddr(change.Addr),
+		Action:          changeAction(change.Action),
+		Reason:          changeReason(change.ActionReason),
+		GeneratedConfig: change.GeneratedConfig,
 	}
+
+	// The order here matters, we want the moved action to take precedence over
+	// the import action. We're basically taking "the most recent action" as the
+	// primary action in the streamed logs. That is to say, that if a resource
+	// is imported and then moved in a single operation then the change for that
+	// resource will be reported as ActionMove while the Importing flag will
+	// still be set to true.
+	//
+	// Since both the moved and imported actions only overwrite a NoOp this
+	// behaviour is consistent across the other actions as well. Something that
+	// is imported and then updated, or moved and then updated, will have the
+	// ActionUpdate as the recognised action for the change.
+
 	if !change.Addr.Equal(change.PrevRunAddr) {
 		if c.Action == ActionNoOp {
 			c.Action = ActionMove
@@ -19,6 +36,12 @@ func NewResourceInstanceChange(change *plans.ResourceInstanceChangeSrc) *Resourc
 		pr := newResourceAddr(change.PrevRunAddr)
 		c.PreviousResource = &pr
 	}
+	if change.Importing != nil {
+		if c.Action == ActionNoOp {
+			c.Action = ActionImport
+		}
+		c.Importing = &Importing{ID: change.Importing.ID}
+	}
 
 	return c
 }
@@ -28,6 +51,8 @@ type ResourceInstanceChange struct {
 	PreviousResource *ResourceAddr `json:"previous_resource,omitempty"`
 	Action           ChangeAction  `json:"action"`
 	Reason           ChangeReason  `json:"reason,omitempty"`
+	Importing        *Importing    `json:"importing,omitempty"`
+	GeneratedConfig  string        `json:"generated_config,omitempty"`
 }
 
 func (c *ResourceInstanceChange) String() string {
@@ -44,6 +69,7 @@ const (
 	ActionUpdate  ChangeAction = "update"
 	ActionReplace ChangeAction = "replace"
 	ActionDelete  ChangeAction = "delete"
+	ActionImport  ChangeAction = "import"
 )
 
 func changeAction(action plans.Action) ChangeAction {
@@ -68,17 +94,22 @@ func changeAction(action plans.Action) ChangeAction {
 type ChangeReason string
 
 const (
-	ReasonNone         ChangeReason = ""
-	ReasonTainted      ChangeReason = "tainted"
-	ReasonRequested    ChangeReason = "requested"
-	ReasonCannotUpdate ChangeReason = "cannot_update"
-	ReasonUnknown      ChangeReason = "unknown"
+	ReasonNone               ChangeReason = ""
+	ReasonTainted            ChangeReason = "tainted"
+	ReasonRequested          ChangeReason = "requested"
+	ReasonReplaceTriggeredBy ChangeReason = "replace_triggered_by"
+	ReasonCannotUpdate       ChangeReason = "cannot_update"
+	ReasonUnknown            ChangeReason = "unknown"
 
 	ReasonDeleteBecauseNoResourceConfig ChangeReason = "delete_because_no_resource_config"
 	ReasonDeleteBecauseWrongRepetition  ChangeReason = "delete_because_wrong_repetition"
 	ReasonDeleteBecauseCountIndex       ChangeReason = "delete_because_count_index"
 	ReasonDeleteBecauseEachKey          ChangeReason = "delete_because_each_key"
 	ReasonDeleteBecauseNoModule         ChangeReason = "delete_because_no_module"
+	ReasonDeleteBecauseNoMoveTarget     ChangeReason = "delete_because_no_move_target"
+	ReasonReadBecauseConfigUnknown      ChangeReason = "read_because_config_unknown"
+	ReasonReadBecauseDependencyPending  ChangeReason = "read_because_dependency_pending"
+	ReasonReadBecauseCheckNested        ChangeReason = "read_because_check_nested"
 )
 
 func changeReason(reason plans.ResourceInstanceChangeActionReason) ChangeReason {
@@ -91,6 +122,8 @@ func changeReason(reason plans.ResourceInstanceChangeActionReason) ChangeReason
 		return ReasonRequested
 	case plans.ResourceInstanceReplaceBecauseCannotUpdate:
 		return ReasonCannotUpdate
+	case plans.ResourceInstanceReplaceByTriggers:
+		return ReasonReplaceTriggeredBy
 	case plans.ResourceInstanceDeleteBecauseNoResourceConfig:
 		return ReasonDeleteBecauseNoResourceConfig
 	case plans.ResourceInstanceDeleteBecauseWrongRepetition:
@@ -101,6 +134,14 @@ func changeReason(reason plans.ResourceInstanceChangeActionReason) ChangeReason
 		return ReasonDeleteBecauseEachKey
 	case plans.ResourceInstanceDeleteBecauseNoModule:
 		return ReasonDeleteBecauseNoModule
+	case plans.ResourceInstanceReadBecauseConfigUnknown:
+		return ReasonReadBecauseConfigUnknown
+	case plans.ResourceInstanceDeleteBecauseNoMoveTarget:
+		return ReasonDeleteBecauseNoMoveTarget
+	case plans.ResourceInstanceReadBecauseDependencyPending:
+		return ReasonReadBecauseDependencyPending
+	case plans.ResourceInstanceReadBecauseCheckNested:
+		return ReasonReadBecauseCheckNested
 	default:
 		// This should never happen, but there's no good way to guarantee
 		// exhaustive handling of the enum, so a generic fall back is better
