@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package command
 
 import (
@@ -20,9 +23,8 @@ import (
 
 func TestWorkspace_createAndChange(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	os.MkdirAll(td, 0755)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	newCmd := &WorkspaceNewCommand{}
@@ -64,9 +66,8 @@ func TestWorkspace_createAndChange(t *testing.T) {
 // This also ensures we switch to the correct env after each call
 func TestWorkspace_createAndList(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	os.MkdirAll(td, 0755)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// make sure a vars file doesn't interfere
@@ -113,9 +114,8 @@ func TestWorkspace_createAndList(t *testing.T) {
 // Create some workspaces and test the show output.
 func TestWorkspace_createAndShow(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	os.MkdirAll(td, 0755)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// make sure a vars file doesn't interfere
@@ -182,9 +182,8 @@ func TestWorkspace_createAndShow(t *testing.T) {
 // Don't allow names that aren't URL safe
 func TestWorkspace_createInvalid(t *testing.T) {
 	// Create a temporary working directory that is empty
-	td := tempDir(t)
+	td := t.TempDir()
 	os.MkdirAll(td, 0755)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	envs := []string{"test_a*", "test_b/foo", "../../../test_c", "好_d"}
@@ -220,9 +219,8 @@ func TestWorkspace_createInvalid(t *testing.T) {
 }
 
 func TestWorkspace_createWithState(t *testing.T) {
-	td := tempDir(t)
+	td := t.TempDir()
 	testCopyDir(t, testFixturePath("inmem-backend"), td)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 	defer inmem.Reset()
 
@@ -291,9 +289,8 @@ func TestWorkspace_createWithState(t *testing.T) {
 }
 
 func TestWorkspace_delete(t *testing.T) {
-	td := tempDir(t)
+	td := t.TempDir()
 	os.MkdirAll(td, 0755)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// create the workspace directories
@@ -345,9 +342,8 @@ func TestWorkspace_delete(t *testing.T) {
 }
 
 func TestWorkspace_deleteInvalid(t *testing.T) {
-	td := tempDir(t)
+	td := t.TempDir()
 	os.MkdirAll(td, 0755)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// choose an invalid workspace name
@@ -378,9 +374,8 @@ func TestWorkspace_deleteInvalid(t *testing.T) {
 }
 
 func TestWorkspace_deleteWithState(t *testing.T) {
-	td := tempDir(t)
+	td := t.TempDir()
 	os.MkdirAll(td, 0755)
-	defer os.RemoveAll(td)
 	defer testChdir(t, td)()
 
 	// create the workspace directories
@@ -443,3 +438,31 @@ func TestWorkspace_deleteWithState(t *testing.T) {
 		t.Fatal("env 'test' still exists!")
 	}
 }
+
+func TestWorkspace_selectWithOrCreate(t *testing.T) {
+	// Create a temporary working directory that is empty
+	td := t.TempDir()
+	os.MkdirAll(td, 0755)
+	defer testChdir(t, td)()
+
+	selectCmd := &WorkspaceSelectCommand{}
+
+	current, _ := selectCmd.Workspace()
+	if current != backend.DefaultStateName {
+		t.Fatal("current workspace should be 'default'")
+	}
+
+	args := []string{"-or-create", "test"}
+	ui := new(cli.MockUi)
+	view, _ := testView(t)
+	selectCmd.Meta = Meta{Ui: ui, View: view}
+	if code := selectCmd.Run(args); code != 0 {
+		t.Fatalf("bad: %d\n\n%s", code, ui.ErrorWriter)
+	}
+
+	current, _ = selectCmd.Workspace()
+	if current != "test" {
+		t.Fatalf("current workspace should be 'test', got %q", current)
+	}
+
+}
