@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package initwd
 
 import (
@@ -10,10 +13,13 @@ import (
 	"path/filepath"
 	"strings"
 
+	"github.com/apparentlymart/go-versions/versions"
 	version "github.com/hashicorp/go-version"
-	"github.com/hashicorp/terraform-config-inspect/tfconfig"
+	"github.com/hashicorp/hcl/v2"
+	"github.com/hashicorp/hcl/v2/hclsyntax"
 	"github.com/hashicorp/terraform/internal/addrs"
-	"github.com/hashicorp/terraform/internal/earlyconfig"
+	"github.com/hashicorp/terraform/internal/configs"
+	"github.com/hashicorp/terraform/internal/configs/configload"
 	"github.com/hashicorp/terraform/internal/getmodules"
 	"github.com/hashicorp/terraform/internal/modsdir"
 	"github.com/hashicorp/terraform/internal/registry"
@@ -24,6 +30,7 @@ import (
 
 type ModuleInstaller struct {
 	modsDir string
+	loader  *configload.Loader
 	reg     *registry.Client
 
 	// The keys in moduleVersions are resolved and trimmed registry source
@@ -40,9 +47,10 @@ type moduleVersion struct {
 	version string
 }
 
-func NewModuleInstaller(modsDir string, reg *registry.Client) *ModuleInstaller {
+func NewModuleInstaller(modsDir string, loader *configload.Loader, reg *registry.Client) *ModuleInstaller {
 	return &ModuleInstaller{
 		modsDir:                 modsDir,
+		loader:                  loader,
 		reg:                     reg,
 		registryPackageVersions: make(map[addrs.ModuleRegistryPackage]*response.ModuleVersions),
 		registryPackageSources:  make(map[moduleVersion]addrs.ModuleSourceRemote),
@@ -69,6 +77,12 @@ func NewModuleInstaller(modsDir string, reg *registry.Client) *ModuleInstaller {
 // needs to replace a directory that is already present with a newly-extracted
 // package.
 //
+// installErrsOnly installs modules but converts validation errors from
+// building the configuration after installation to warnings. This is used by
+// commands like `get` or `init -from-module` where the established behavior
+// was only to install the requested module, and extra validation can break
+// compatibility.
+//
 // If the returned diagnostics contains errors then the module installation
 // may have wholly or partially completed. Modules must be loaded in order
 // to find their dependencies, so this function does many of the same checks
@@ -77,12 +91,23 @@ func NewModuleInstaller(modsDir string, reg *registry.Client) *ModuleInstaller {
 // If successful (the returned diagnostics contains no errors) then the
 // first return value is the early configuration tree that was constructed by
 // the installation process.
-func (i *ModuleInstaller) InstallModules(ctx context.Context, rootDir string, upgrade bool, hooks ModuleInstallHooks) (*earlyconfig.Config, tfdiags.Diagnostics) {
+func (i *ModuleInstaller) InstallModules(ctx context.Context, rootDir, testsDir string, upgrade, installErrsOnly bool, hooks ModuleInstallHooks) (*configs.Config, tfdiags.Diagnostics) {
 	log.Printf("[TRACE] ModuleInstaller: installing child modules for %s into %s", rootDir, i.modsDir)
+	var diags tfdiags.Diagnostics
 
-	rootMod, diags := earlyconfig.LoadModule(rootDir)
+	rootMod, mDiags := i.loader.Parser().LoadConfigDirWithTests(rootDir, testsDir)
 	if rootMod == nil {
+		// We drop the diagnostics here because we only want to report module
+		// loading errors after checking the core version constraints, which we
+		// can only do if the module can be at least partially loaded.
 		return nil, diags
+	} else if vDiags := rootMod.CheckCoreVersionRequirements(nil, nil); vDiags.HasErrors() {
+		// If the core version requirements are not met, we drop any other
+		// diagnostics, as they may reflect language changes from future
+		// Terraform versions.
+		diags = diags.Append(vDiags)
+	} else {
+		diags = diags.Append(mDiags)
 	}
 
 	manifest, err := modsdir.ReadManifestSnapshotForDir(i.modsDir)
@@ -96,14 +121,6 @@ func (i *ModuleInstaller) InstallModules(ctx context.Context, rootDir string, up
 	}
 
 	fetcher := getmodules.NewPackageFetcher()
-	cfg, instDiags := i.installDescendentModules(ctx, rootMod, rootDir, manifest, upgrade, hooks, fetcher)
-	diags = append(diags, instDiags...)
-
-	return cfg, diags
-}
-
-func (i *ModuleInstaller) installDescendentModules(ctx context.Context, rootMod *tfconfig.Module, rootDir string, manifest modsdir.Manifest, upgrade bool, hooks ModuleInstallHooks, fetcher *getmodules.PackageFetcher) (*earlyconfig.Config, tfdiags.Diagnostics) {
-	var diags tfdiags.Diagnostics
 
 	if hooks == nil {
 		// Use our no-op implementation as a placeholder
@@ -116,9 +133,41 @@ func (i *ModuleInstaller) installDescendentModules(ctx context.Context, rootMod
 		Key: "",
 		Dir: rootDir,
 	}
+	walker := i.moduleInstallWalker(ctx, manifest, upgrade, hooks, fetcher)
 
-	cfg, cDiags := earlyconfig.BuildConfig(rootMod, earlyconfig.ModuleWalkerFunc(
-		func(req *earlyconfig.ModuleRequest) (*tfconfig.Module, *version.Version, tfdiags.Diagnostics) {
+	cfg, instDiags := i.installDescendentModules(rootMod, manifest, walker, installErrsOnly)
+	diags = append(diags, instDiags...)
+
+	return cfg, diags
+}
+
+func (i *ModuleInstaller) moduleInstallWalker(ctx context.Context, manifest modsdir.Manifest, upgrade bool, hooks ModuleInstallHooks, fetcher *getmodules.PackageFetcher) configs.ModuleWalker {
+	return configs.ModuleWalkerFunc(
+		func(req *configs.ModuleRequest) (*configs.Module, *version.Version, hcl.Diagnostics) {
+			var diags hcl.Diagnostics
+
+			if req.SourceAddr == nil {
+				// If the parent module failed to parse the module source
+				// address, we can't load it here. Return nothing as the parent
+				// module's diagnostics should explain this.
+				return nil, nil, diags
+			}
+
+			if req.Name == "" {
+				// An empty string for a module instance name breaks our
+				// manifest map, which uses that to indicate the root module.
+				// Because we descend into modules which have errors, we need
+				// to look out for this case, but the config loader's
+				// diagnostics will report the error later.
+				return nil, nil, diags
+			}
+
+			if !hclsyntax.ValidIdentifier(req.Name) {
+				// A module with an invalid name shouldn't be installed at all. This is
+				// mostly a concern for remote modules, since we need to be able to convert
+				// the name to a valid path.
+				return nil, nil, diags
+			}
 
 			key := manifest.ModuleKey(req.Path)
 			instPath := i.packageInstallPath(req.Path)
@@ -137,8 +186,8 @@ func (i *ModuleInstaller) installDescendentModules(ctx context.Context, rootMod
 				case record.SourceAddr != req.SourceAddr.String():
 					log.Printf("[TRACE] ModuleInstaller: %s source address has changed from %q to %q", key, record.SourceAddr, req.SourceAddr)
 					replace = true
-				case record.Version != nil && !req.VersionConstraints.Check(record.Version):
-					log.Printf("[TRACE] ModuleInstaller: %s version %s no longer compatible with constraints %s", key, record.Version, req.VersionConstraints)
+				case record.Version != nil && !req.VersionConstraint.Required.Check(record.Version):
+					log.Printf("[TRACE] ModuleInstaller: %s version %s no longer compatible with constraints %s", key, record.Version, req.VersionConstraint.Required)
 					replace = true
 				}
 			}
@@ -172,14 +221,14 @@ func (i *ModuleInstaller) installDescendentModules(ctx context.Context, rootMod
 				err := os.RemoveAll(instPath)
 				if err != nil && !os.IsNotExist(err) {
 					log.Printf("[TRACE] ModuleInstaller: failed to remove %s: %s", key, err)
-					diags = diags.Append(tfdiags.Sourceless(
-						tfdiags.Error,
-						"Failed to remove local module cache",
-						fmt.Sprintf(
+					diags = diags.Append(&hcl.Diagnostic{
+						Severity: hcl.DiagError,
+						Summary:  "Failed to remove local module cache",
+						Detail: fmt.Sprintf(
 							"Terraform tried to remove %s in order to reinstall this module, but encountered an error: %s",
 							instPath, err,
 						),
-					))
+					})
 					return nil, nil, diags
 				}
 			} else {
@@ -188,8 +237,19 @@ func (i *ModuleInstaller) installDescendentModules(ctx context.Context, rootMod
 				// keep our existing record.
 				info, err := os.Stat(record.Dir)
 				if err == nil && info.IsDir() {
-					mod, mDiags := earlyconfig.LoadModule(record.Dir)
-					diags = diags.Append(mDiags)
+					mod, mDiags := i.loader.Parser().LoadConfigDir(record.Dir)
+					if mod == nil {
+						// nil indicates an unreadable module, which should never happen,
+						// so we return the full loader diagnostics here.
+						diags = diags.Extend(mDiags)
+					} else if vDiags := mod.CheckCoreVersionRequirements(req.Path, req.SourceAddr); vDiags.HasErrors() {
+						// If the core version requirements are not met, we drop any other
+						// diagnostics, as they may reflect language changes from future
+						// Terraform versions.
+						diags = diags.Extend(vDiags)
+					} else {
+						diags = diags.Extend(mDiags)
+					}
 
 					log.Printf("[TRACE] ModuleInstaller: Module installer: %s %s already installed in %s", key, record.Version, record.Dir)
 					return mod, record.Version, diags
@@ -226,10 +286,49 @@ func (i *ModuleInstaller) installDescendentModules(ctx context.Context, rootMod
 				// of addrs.ModuleSource.
 				panic(fmt.Sprintf("unsupported module source address %#v", addr))
 			}
-
 		},
-	))
-	diags = append(diags, cDiags...)
+	)
+}
+
+func (i *ModuleInstaller) installDescendentModules(rootMod *configs.Module, manifest modsdir.Manifest, installWalker configs.ModuleWalker, installErrsOnly bool) (*configs.Config, tfdiags.Diagnostics) {
+	var diags tfdiags.Diagnostics
+
+	// When attempting to initialize the current directory with a module
+	// source, some use cases may want to ignore configuration errors from the
+	// building of the entire configuration structure, but we still need to
+	// capture installation errors. Because the actual module installation
+	// happens in the ModuleWalkFunc callback while building the config, we
+	// need to create a closure to capture the installation diagnostics
+	// separately.
+	var instDiags hcl.Diagnostics
+	walker := installWalker
+	if installErrsOnly {
+		walker = configs.ModuleWalkerFunc(func(req *configs.ModuleRequest) (*configs.Module, *version.Version, hcl.Diagnostics) {
+			mod, version, diags := installWalker.LoadModule(req)
+			instDiags = instDiags.Extend(diags)
+			return mod, version, diags
+		})
+	}
+
+	cfg, cDiags := configs.BuildConfig(rootMod, walker)
+	diags = diags.Append(cDiags)
+	if installErrsOnly {
+		// We can't continue if there was an error during installation, but
+		// return all diagnostics in case there happens to be anything else
+		// useful when debugging the problem. Any instDiags will be included in
+		// diags already.
+		if instDiags.HasErrors() {
+			return cfg, diags
+		}
+
+		// If there are any errors here, they must be only from building the
+		// config structures. We don't want to block initialization at this
+		// point, so convert these into warnings. Any actual errors in the
+		// configuration will be raised as soon as the config is loaded again.
+		// We continue below because writing the manifest is required to finish
+		// module installation.
+		diags = tfdiags.OverrideAll(diags, tfdiags.Warning, nil)
+	}
 
 	err := manifest.WriteSnapshotToDir(i.modsDir)
 	if err != nil {
@@ -243,8 +342,8 @@ func (i *ModuleInstaller) installDescendentModules(ctx context.Context, rootMod
 	return cfg, diags
 }
 
-func (i *ModuleInstaller) installLocalModule(req *earlyconfig.ModuleRequest, key string, manifest modsdir.Manifest, hooks ModuleInstallHooks) (*tfconfig.Module, tfdiags.Diagnostics) {
-	var diags tfdiags.Diagnostics
+func (i *ModuleInstaller) installLocalModule(req *configs.ModuleRequest, key string, manifest modsdir.Manifest, hooks ModuleInstallHooks) (*configs.Module, hcl.Diagnostics) {
+	var diags hcl.Diagnostics
 
 	parentKey := manifest.ModuleKey(req.Parent.Path)
 	parentRecord, recorded := manifest[parentKey]
@@ -253,12 +352,13 @@ func (i *ModuleInstaller) installLocalModule(req *earlyconfig.ModuleRequest, key
 		panic(fmt.Errorf("missing manifest record for parent module %s", parentKey))
 	}
 
-	if len(req.VersionConstraints) != 0 {
-		diags = diags.Append(tfdiags.Sourceless(
-			tfdiags.Error,
-			"Invalid version constraint",
-			fmt.Sprintf("Cannot apply a version constraint to module %q (at %s:%d) because it has a relative local path.", req.Name, req.CallPos.Filename, req.CallPos.Line),
-		))
+	if len(req.VersionConstraint.Required) != 0 {
+		diags = diags.Append(&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Invalid version constraint",
+			Detail:   fmt.Sprintf("Cannot apply a version constraint to module %q (at %s:%d) because it has a relative local path.", req.Name, req.CallRange.Filename, req.CallRange.Start.Line),
+			Subject:  req.CallRange.Ptr(),
+		})
 	}
 
 	// For local sources we don't actually need to modify the
@@ -270,25 +370,31 @@ func (i *ModuleInstaller) installLocalModule(req *earlyconfig.ModuleRequest, key
 	// it is possible that the local directory is a symlink
 	newDir, err := filepath.EvalSymlinks(newDir)
 	if err != nil {
-		diags = diags.Append(tfdiags.Sourceless(
-			tfdiags.Error,
-			"Unreadable module directory",
-			fmt.Sprintf("Unable to evaluate directory symlink: %s", err.Error()),
-		))
+		diags = diags.Append(&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Unreadable module directory",
+			Detail:   fmt.Sprintf("Unable to evaluate directory symlink: %s", err.Error()),
+		})
 	}
 
-	mod, mDiags := earlyconfig.LoadModule(newDir)
+	// Finally we are ready to try actually loading the module.
+	mod, mDiags := i.loader.Parser().LoadConfigDir(newDir)
 	if mod == nil {
 		// nil indicates missing or unreadable directory, so we'll
 		// discard the returned diags and return a more specific
 		// error message here.
-		diags = diags.Append(tfdiags.Sourceless(
-			tfdiags.Error,
-			"Unreadable module directory",
-			fmt.Sprintf("The directory %s could not be read for module %q at %s:%d.", newDir, req.Name, req.CallPos.Filename, req.CallPos.Line),
-		))
+		diags = diags.Append(&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Unreadable module directory",
+			Detail:   fmt.Sprintf("The directory %s could not be read for module %q at %s:%d.", newDir, req.Name, req.CallRange.Filename, req.CallRange.Start.Line),
+		})
+	} else if vDiags := mod.CheckCoreVersionRequirements(req.Path, req.SourceAddr); vDiags.HasErrors() {
+		// If the core version requirements are not met, we drop any other
+		// diagnostics, as they may reflect language changes from future
+		// Terraform versions.
+		diags = diags.Extend(vDiags)
 	} else {
-		diags = diags.Append(mDiags)
+		diags = diags.Extend(mDiags)
 	}
 
 	// Note the local location in our manifest.
@@ -303,10 +409,10 @@ func (i *ModuleInstaller) installLocalModule(req *earlyconfig.ModuleRequest, key
 	return mod, diags
 }
 
-func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *earlyconfig.ModuleRequest, key string, instPath string, addr addrs.ModuleSourceRegistry, manifest modsdir.Manifest, hooks ModuleInstallHooks, fetcher *getmodules.PackageFetcher) (*tfconfig.Module, *version.Version, tfdiags.Diagnostics) {
-	var diags tfdiags.Diagnostics
+func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *configs.ModuleRequest, key string, instPath string, addr addrs.ModuleSourceRegistry, manifest modsdir.Manifest, hooks ModuleInstallHooks, fetcher *getmodules.PackageFetcher) (*configs.Module, *version.Version, hcl.Diagnostics) {
+	var diags hcl.Diagnostics
 
-	hostname := addr.PackageAddr.Host
+	hostname := addr.Package.Host
 	reg := i.reg
 	var resp *response.ModuleVersions
 	var exists bool
@@ -314,7 +420,7 @@ func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *earlyc
 	// A registry entry isn't _really_ a module package, but we'll pretend it's
 	// one for the sake of this reporting by just trimming off any source
 	// directory.
-	packageAddr := addr.PackageAddr
+	packageAddr := addr.Package
 
 	// Our registry client is still using the legacy model of addresses, so
 	// we'll shim it here for now.
@@ -329,23 +435,25 @@ func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *earlyc
 		resp, err = reg.ModuleVersions(ctx, regsrcAddr)
 		if err != nil {
 			if registry.IsModuleNotFound(err) {
-				diags = diags.Append(tfdiags.Sourceless(
-					tfdiags.Error,
-					"Module not found",
-					fmt.Sprintf("Module %q (from %s:%d) cannot be found in the module registry at %s.", req.Name, req.CallPos.Filename, req.CallPos.Line, hostname),
-				))
+				diags = diags.Append(&hcl.Diagnostic{
+					Severity: hcl.DiagError,
+					Summary:  "Module not found",
+					Detail:   fmt.Sprintf("Module %q (from %s:%d) cannot be found in the module registry at %s.", req.Name, req.CallRange.Filename, req.CallRange.Start.Line, hostname),
+					Subject:  req.CallRange.Ptr(),
+				})
 			} else if errors.Is(err, context.Canceled) {
-				diags = diags.Append(tfdiags.Sourceless(
-					tfdiags.Error,
-					"Module installation was interrupted",
-					fmt.Sprintf("Received interrupt signal while retrieving available versions for module %q.", req.Name),
-				))
+				diags = diags.Append(&hcl.Diagnostic{
+					Severity: hcl.DiagError,
+					Summary:  "Module installation was interrupted",
+					Detail:   fmt.Sprintf("Received interrupt signal while retrieving available versions for module %q.", req.Name),
+				})
 			} else {
-				diags = diags.Append(tfdiags.Sourceless(
-					tfdiags.Error,
-					"Error accessing remote module registry",
-					fmt.Sprintf("Failed to retrieve available versions for module %q (%s:%d) from %s: %s.", req.Name, req.CallPos.Filename, req.CallPos.Line, hostname, err),
-				))
+				diags = diags.Append(&hcl.Diagnostic{
+					Severity: hcl.DiagError,
+					Summary:  "Error accessing remote module registry",
+					Detail:   fmt.Sprintf("Failed to retrieve available versions for module %q (%s:%d) from %s: %s.", req.Name, req.CallRange.Filename, req.CallRange.Start.Line, hostname, err),
+					Subject:  req.CallRange.Ptr(),
+				})
 			}
 			return nil, nil, diags
 		}
@@ -359,11 +467,12 @@ func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *earlyc
 	if len(resp.Modules) < 1 {
 		// Should never happen, but since this is a remote service that may
 		// be implemented by third-parties we will handle it gracefully.
-		diags = diags.Append(tfdiags.Sourceless(
-			tfdiags.Error,
-			"Invalid response from remote module registry",
-			fmt.Sprintf("The registry at %s returned an invalid response when Terraform requested available versions for module %q (%s:%d).", hostname, req.Name, req.CallPos.Filename, req.CallPos.Line),
-		))
+		diags = diags.Append(&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Invalid response from remote module registry",
+			Detail:   fmt.Sprintf("The registry at %s returned an invalid response when Terraform requested available versions for module %q (%s:%d).", hostname, req.Name, req.CallRange.Filename, req.CallRange.Start.Line),
+			Subject:  req.CallRange.Ptr(),
+		})
 		return nil, nil, diags
 	}
 
@@ -377,26 +486,63 @@ func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *earlyc
 			// Should never happen if the registry server is compliant with
 			// the protocol, but we'll warn if not to assist someone who
 			// might be developing a module registry server.
-			diags = diags.Append(tfdiags.Sourceless(
-				tfdiags.Warning,
-				"Invalid response from remote module registry",
-				fmt.Sprintf("The registry at %s returned an invalid version string %q for module %q (%s:%d), which Terraform ignored.", hostname, mv.Version, req.Name, req.CallPos.Filename, req.CallPos.Line),
-			))
+			diags = diags.Append(&hcl.Diagnostic{
+				Severity: hcl.DiagWarning,
+				Summary:  "Invalid response from remote module registry",
+				Detail:   fmt.Sprintf("The registry at %s returned an invalid version string %q for module %q (%s:%d), which Terraform ignored.", hostname, mv.Version, req.Name, req.CallRange.Filename, req.CallRange.Start.Line),
+				Subject:  req.CallRange.Ptr(),
+			})
 			continue
 		}
 
 		// If we've found a pre-release version then we'll ignore it unless
 		// it was exactly requested.
-		if v.Prerelease() != "" && req.VersionConstraints.String() != v.String() {
-			log.Printf("[TRACE] ModuleInstaller: %s ignoring %s because it is a pre-release and was not requested exactly", key, v)
-			continue
+		//
+		// The prerelease checking will be handled by a different library for
+		// 2 reasons. First, this other library automatically includes the
+		// "prerelease versions must be exactly requested" behaviour that we are
+		// looking for. Second, this other library is used to handle all version
+		// constraints for the provider logic and this is the first step to
+		// making the module and provider version logic match.
+		if v.Prerelease() != "" {
+			// At this point all versions published by the module with
+			// prerelease metadata will be checked. Users may not have even
+			// requested this prerelease so don't print lots of unnecessary #
+			// warnings.
+			acceptableVersions, err := versions.MeetingConstraintsString(req.VersionConstraint.Required.String())
+			if err != nil {
+				log.Printf("[WARN] ModuleInstaller: %s ignoring %s because the version constraints (%s) could not be parsed: %s", key, v, req.VersionConstraint.Required.String(), err.Error())
+				continue
+			}
+
+			// Validate the version is also readable by the other versions
+			// library.
+			version, err := versions.ParseVersion(v.String())
+			if err != nil {
+				log.Printf("[WARN] ModuleInstaller: %s ignoring %s because the version (%s) reported by the module could not be parsed: %s", key, v, v.String(), err.Error())
+				continue
+			}
+
+			// Finally, check if the prerelease is acceptable to version. As
+			// highlighted previously, we go through all of this because the
+			// apparentlymart/go-versions library handles prerelease constraints
+			// in the apporach we want to.
+			if !acceptableVersions.Has(version) {
+				log.Printf("[TRACE] ModuleInstaller: %s ignoring %s because it is a pre-release and was not requested exactly", key, v)
+				continue
+			}
+
+			// If we reach here, it means this prerelease version was exactly
+			// requested according to the extra constraints of this library.
+			// We fall through and allow the other library to also validate it
+			// for consistency.
 		}
 
 		if latestVersion == nil || v.GreaterThan(latestVersion) {
 			latestVersion = v
 		}
 
-		if req.VersionConstraints.Check(v) {
+		if req.VersionConstraint.Required.Check(v) {
 			if latestMatch == nil || v.GreaterThan(latestMatch) {
 				latestMatch = v
 			}
@@ -404,20 +550,22 @@ func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *earlyc
 	}
 
 	if latestVersion == nil {
-		diags = diags.Append(tfdiags.Sourceless(
-			tfdiags.Error,
-			"Module has no versions",
-			fmt.Sprintf("Module %q (%s:%d) has no versions available on %s.", addr, req.CallPos.Filename, req.CallPos.Line, hostname),
-		))
+		diags = diags.Append(&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Module has no versions",
+			Detail:   fmt.Sprintf("Module %q (%s:%d) has no versions available on %s.", addr, req.CallRange.Filename, req.CallRange.Start.Line, hostname),
+			Subject:  req.CallRange.Ptr(),
+		})
 		return nil, nil, diags
 	}
 
 	if latestMatch == nil {
-		diags = diags.Append(tfdiags.Sourceless(
-			tfdiags.Error,
-			"Unresolvable module version constraint",
-			fmt.Sprintf("There is no available version of module %q (%s:%d) which matches the given version constraint. The newest available version is %s.", addr, req.CallPos.Filename, req.CallPos.Line, latestVersion),
-		))
+		diags = diags.Append(&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Unresolvable module version constraint",
+			Detail:   fmt.Sprintf("There is no available version of module %q (%s:%d) which matches the given version constraint. The newest available version is %s.", addr, req.CallRange.Filename, req.CallRange.Start.Line, latestVersion),
+			Subject:  req.CallRange.Ptr(),
+		})
 		return nil, nil, diags
 	}
 
@@ -434,20 +582,20 @@ func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *earlyc
 		realAddrRaw, err := reg.ModuleLocation(ctx, regsrcAddr, latestMatch.String())
 		if err != nil {
 			log.Printf("[ERROR] %s from %s %s: %s", key, addr, latestMatch, err)
-			diags = diags.Append(tfdiags.Sourceless(
-				tfdiags.Error,
-				"Error accessing remote module registry",
-				fmt.Sprintf("Failed to retrieve a download URL for %s %s from %s: %s", addr, latestMatch, hostname, err),
-			))
+			diags = diags.Append(&hcl.Diagnostic{
+				Severity: hcl.DiagError,
+				Summary:  "Error accessing remote module registry",
+				Detail:   fmt.Sprintf("Failed to retrieve a download URL for %s %s from %s: %s", addr, latestMatch, hostname, err),
+			})
 			return nil, nil, diags
 		}
 		realAddr, err := addrs.ParseModuleSource(realAddrRaw)
 		if err != nil {
-			diags = diags.Append(tfdiags.Sourceless(
-				tfdiags.Error,
-				"Invalid package location from module registry",
-				fmt.Sprintf("Module registry %s returned invalid source location %q for %s %s: %s.", hostname, realAddrRaw, addr, latestMatch, err),
-			))
+			diags = diags.Append(&hcl.Diagnostic{
+				Severity: hcl.DiagError,
+				Summary:  "Invalid package location from module registry",
+				Detail:   fmt.Sprintf("Module registry %s returned invalid source location %q for %s %s: %s.", hostname, realAddrRaw, addr, latestMatch, err),
+			})
 			return nil, nil, diags
 		}
 		switch realAddr := realAddr.(type) {
@@ -458,26 +606,26 @@ func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *earlyc
 		case addrs.ModuleSourceRemote:
 			i.registryPackageSources[moduleAddr] = realAddr
 		default:
-			diags = diags.Append(tfdiags.Sourceless(
-				tfdiags.Error,
-				"Invalid package location from module registry",
-				fmt.Sprintf("Module registry %s returned invalid source location %q for %s %s: must be a direct remote package address.", hostname, realAddrRaw, addr, latestMatch),
-			))
+			diags = diags.Append(&hcl.Diagnostic{
+				Severity: hcl.DiagError,
+				Summary:  "Invalid package location from module registry",
+				Detail:   fmt.Sprintf("Module registry %s returned invalid source location %q for %s %s: must be a direct remote package address.", hostname, realAddrRaw, addr, latestMatch),
+			})
 			return nil, nil, diags
 		}
 	}
 
 	dlAddr := i.registryPackageSources[moduleAddr]
 
-	log.Printf("[TRACE] ModuleInstaller: %s %s %s is available at %q", key, packageAddr, latestMatch, dlAddr.PackageAddr)
+	log.Printf("[TRACE] ModuleInstaller: %s %s %s is available at %q", key, packageAddr, latestMatch, dlAddr.Package)
 
-	err := fetcher.FetchPackage(ctx, instPath, dlAddr.PackageAddr.String())
+	err := fetcher.FetchPackage(ctx, instPath, dlAddr.Package.String())
 	if errors.Is(err, context.Canceled) {
-		diags = diags.Append(tfdiags.Sourceless(
-			tfdiags.Error,
-			"Module download was interrupted",
-			fmt.Sprintf("Interrupt signal received when downloading module %s.", addr),
-		))
+		diags = diags.Append(&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Module download was interrupted",
+			Detail:   fmt.Sprintf("Interrupt signal received when downloading module %s.", addr),
+		})
 		return nil, nil, diags
 	}
 	if err != nil {
@@ -486,15 +634,16 @@ func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *earlyc
 		// we have no way to recognize any specific errors to improve them
 		// and masking the error entirely would hide valuable diagnostic
 		// information from the user.
-		diags = diags.Append(tfdiags.Sourceless(
-			tfdiags.Error,
-			"Failed to download module",
-			fmt.Sprintf("Could not download module %q (%s:%d) source code from %q: %s.", req.Name, req.CallPos.Filename, req.CallPos.Line, dlAddr, err),
-		))
+		diags = diags.Append(&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Failed to download module",
+			Detail:   fmt.Sprintf("Could not download module %q (%s:%d) source code from %q: %s.", req.Name, req.CallRange.Filename, req.CallRange.Start.Line, dlAddr, err),
+			Subject:  req.CallRange.Ptr(),
+		})
 		return nil, nil, diags
 	}
 
-	log.Printf("[TRACE] ModuleInstaller: %s %q was downloaded to %s", key, dlAddr.PackageAddr, instPath)
+	log.Printf("[TRACE] ModuleInstaller: %s %q was downloaded to %s", key, dlAddr.Package, instPath)
 
 	// Incorporate any subdir information from the original path into the
 	// address returned by the registry in order to find the final directory
@@ -506,20 +655,25 @@ func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *earlyc
 	log.Printf("[TRACE] ModuleInstaller: %s should now be at %s", key, modDir)
 
 	// Finally we are ready to try actually loading the module.
-	mod, mDiags := earlyconfig.LoadModule(modDir)
+	mod, mDiags := i.loader.Parser().LoadConfigDir(modDir)
 	if mod == nil {
 		// nil indicates missing or unreadable directory, so we'll
 		// discard the returned diags and return a more specific
 		// error message here. For registry modules this actually
 		// indicates a bug in the code above, since it's not the
 		// user's responsibility to create the directory in this case.
-		diags = diags.Append(tfdiags.Sourceless(
-			tfdiags.Error,
-			"Unreadable module directory",
-			fmt.Sprintf("The directory %s could not be read. This is a bug in Terraform and should be reported.", modDir),
-		))
+		diags = diags.Append(&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Unreadable module directory",
+			Detail:   fmt.Sprintf("The directory %s could not be read. This is a bug in Terraform and should be reported.", modDir),
+		})
+	} else if vDiags := mod.CheckCoreVersionRequirements(req.Path, req.SourceAddr); vDiags.HasErrors() {
+		// If the core version requirements are not met, we drop any other
+		// diagnostics, as they may reflect language changes from future
+		// Terraform versions.
+		diags = diags.Extend(vDiags)
 	} else {
-		diags = append(diags, mDiags...)
+		diags = diags.Extend(mDiags)
 	}
 
 	// Note the local location in our manifest.
@@ -535,20 +689,21 @@ func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *earlyc
 	return mod, latestMatch, diags
 }
 
-func (i *ModuleInstaller) installGoGetterModule(ctx context.Context, req *earlyconfig.ModuleRequest, key string, instPath string, manifest modsdir.Manifest, hooks ModuleInstallHooks, fetcher *getmodules.PackageFetcher) (*tfconfig.Module, tfdiags.Diagnostics) {
-	var diags tfdiags.Diagnostics
+func (i *ModuleInstaller) installGoGetterModule(ctx context.Context, req *configs.ModuleRequest, key string, instPath string, manifest modsdir.Manifest, hooks ModuleInstallHooks, fetcher *getmodules.PackageFetcher) (*configs.Module, hcl.Diagnostics) {
+	var diags hcl.Diagnostics
 
 	// Report up to the caller that we're about to start downloading.
 	addr := req.SourceAddr.(addrs.ModuleSourceRemote)
-	packageAddr := addr.PackageAddr
+	packageAddr := addr.Package
 	hooks.Download(key, packageAddr.String(), nil)
 
-	if len(req.VersionConstraints) != 0 {
-		diags = diags.Append(tfdiags.Sourceless(
-			tfdiags.Error,
-			"Invalid version constraint",
-			fmt.Sprintf("Cannot apply a version constraint to module %q (at %s:%d) because it has a non Registry URL.", req.Name, req.CallPos.Filename, req.CallPos.Line),
-		))
+	if len(req.VersionConstraint.Required) != 0 {
+		diags = diags.Append(&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Invalid version constraint",
+			Detail:   fmt.Sprintf("Cannot apply a version constraint to module %q (at %s:%d) because it doesn't come from a module registry.", req.Name, req.CallRange.Filename, req.CallRange.Start.Line),
+			Subject:  req.CallRange.Ptr(),
+		})
 		return nil, diags
 	}
 
@@ -561,51 +716,65 @@ func (i *ModuleInstaller) installGoGetterModule(ctx context.Context, req *earlyc
 				"[TRACE] ModuleInstaller: %s looks like a local path but is missing ./ or ../",
 				req.SourceAddr,
 			)
-			diags = diags.Append(tfdiags.Sourceless(
-				tfdiags.Error,
-				"Module not found",
-				fmt.Sprintf(
+			diags = diags.Append(&hcl.Diagnostic{
+				Severity: hcl.DiagError,
+				Summary:  "Module not found",
+				Detail: fmt.Sprintf(
 					"The module address %q could not be resolved.\n\n"+
 						"If you intended this as a path relative to the current "+
 						"module, use \"./%s\" instead. The \"./\" prefix "+
 						"indicates that the address is a relative filesystem path.",
 					req.SourceAddr, req.SourceAddr,
 				),
-			))
+			})
 		} else {
 			// Errors returned by go-getter have very inconsistent quality as
 			// end-user error messages, but for now we're accepting that because
 			// we have no way to recognize any specific errors to improve them
 			// and masking the error entirely would hide valuable diagnostic
 			// information from the user.
-			diags = diags.Append(tfdiags.Sourceless(
-				tfdiags.Error,
-				"Failed to download module",
-				fmt.Sprintf("Could not download module %q (%s:%d) source code from %q: %s", req.Name, req.CallPos.Filename, req.CallPos.Line, packageAddr, err),
-			))
+			diags = diags.Append(&hcl.Diagnostic{
+				Severity: hcl.DiagError,
+				Summary:  "Failed to download module",
+				Detail:   fmt.Sprintf("Could not download module %q (%s:%d) source code from %q: %s", req.Name, req.CallRange.Filename, req.CallRange.Start.Line, packageAddr, err),
+				Subject:  req.CallRange.Ptr(),
+			})
 		}
 		return nil, diags
 	}
 
-	subDir := filepath.FromSlash(addr.Subdir)
-	modDir := filepath.Join(instPath, subDir)
+	modDir, err := getmodules.ExpandSubdirGlobs(instPath, addr.Subdir)
+	if err != nil {
+		diags = diags.Append(&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Failed to expand subdir globs",
+			Detail:   err.Error(),
+		})
+		return nil, diags
+	}
 
 	log.Printf("[TRACE] ModuleInstaller: %s %q was downloaded to %s", key, addr, modDir)
 
-	mod, mDiags := earlyconfig.LoadModule(modDir)
+	// Finally we are ready to try actually loading the module.
+	mod, mDiags := i.loader.Parser().LoadConfigDir(modDir)
 	if mod == nil {
 		// nil indicates missing or unreadable directory, so we'll
 		// discard the returned diags and return a more specific
 		// error message here. For go-getter modules this actually
 		// indicates a bug in the code above, since it's not the
 		// user's responsibility to create the directory in this case.
-		diags = diags.Append(tfdiags.Sourceless(
-			tfdiags.Error,
-			"Unreadable module directory",
-			fmt.Sprintf("The directory %s could not be read. This is a bug in Terraform and should be reported.", modDir),
-		))
+		diags = diags.Append(&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Unreadable module directory",
+			Detail:   fmt.Sprintf("The directory %s could not be read. This is a bug in Terraform and should be reported.", modDir),
+		})
+	} else if vDiags := mod.CheckCoreVersionRequirements(req.Path, req.SourceAddr); vDiags.HasErrors() {
+		// If the core version requirements are not met, we drop any other
+		// diagnostics, as they may reflect language changes from future
+		// Terraform versions.
+		diags = diags.Extend(vDiags)
 	} else {
-		diags = append(diags, mDiags...)
+		diags = diags.Extend(mDiags)
 	}
 
 	// Note the local location in our manifest.
@@ -633,7 +802,7 @@ func (i *ModuleInstaller) packageInstallPath(modulePath addrs.Module) string {
 //
 // This function's behavior is only reasonable for errors returned from the
 // ModuleInstaller.installLocalModule function.
-func maybeImproveLocalInstallError(req *earlyconfig.ModuleRequest, diags tfdiags.Diagnostics) tfdiags.Diagnostics {
+func maybeImproveLocalInstallError(req *configs.ModuleRequest, diags hcl.Diagnostics) hcl.Diagnostics {
 	if !diags.HasErrors() {
 		return diags
 	}
@@ -668,7 +837,7 @@ func maybeImproveLocalInstallError(req *earlyconfig.ModuleRequest, diags tfdiags
 		Path:       req.Path,
 		SourceAddr: req.SourceAddr,
 	})
-	current := req.Parent // an earlyconfig.Config where Children isn't populated yet
+	current := req.Parent // a configs.Config where Children isn't populated yet
 	for {
 		if current == nil || current.SourceAddr == nil {
 			// We've reached the root module, in which case we aren't
@@ -712,11 +881,11 @@ func maybeImproveLocalInstallError(req *earlyconfig.ModuleRequest, diags tfdiags
 		if !strings.HasPrefix(nextPath, prefix) { // ESCAPED!
 			escapeeAddr := step.Path.String()
 
-			var newDiags tfdiags.Diagnostics
+			var newDiags hcl.Diagnostics
 
 			// First we'll copy over any non-error diagnostics from the source diags
 			for _, diag := range diags {
-				if diag.Severity() != tfdiags.Error {
+				if diag.Severity != hcl.DiagError {
 					newDiags = newDiags.Append(diag)
 				}
 			}
@@ -731,14 +900,14 @@ func maybeImproveLocalInstallError(req *earlyconfig.ModuleRequest, diags tfdiags
 				// about it.
 				suggestion = "\n\nTerraform treats absolute filesystem paths as external modules which establish a new module package. To treat this directory as part of the same package as its caller, use a local path starting with either \"./\" or \"../\"."
 			}
-			newDiags = newDiags.Append(tfdiags.Sourceless(
-				tfdiags.Error,
-				"Local module path escapes module package",
-				fmt.Sprintf(
+			newDiags = newDiags.Append(&hcl.Diagnostic{
+				Severity: hcl.DiagError,
+				Summary:  "Local module path escapes module package",
+				Detail: fmt.Sprintf(
 					"The given source directory for %s would be outside of its containing package %q. Local source addresses starting with \"../\" must stay within the same package that the calling module belongs to.%s",
 					escapeeAddr, packageAddr, suggestion,
 				),
-			))
+			})
 
 			return newDiags
 		}
@@ -758,7 +927,7 @@ func splitAddrSubdir(addr addrs.ModuleSource) (string, string) {
 		addr.Subdir = ""
 		return addr.String(), subDir
 	case addrs.ModuleSourceRemote:
-		return addr.PackageAddr.String(), addr.Subdir
+		return addr.Package.String(), addr.Subdir
 	case nil:
 		panic("splitAddrSubdir on nil addrs.ModuleSource")
 	default:
