@@ -1,8 +1,11 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package planfile
 
 import (
-	"io/ioutil"
 	"path/filepath"
+	"strings"
 	"testing"
 
 	"github.com/google/go-cmp/cmp"
@@ -63,6 +66,7 @@ func TestRoundtrip(t *testing.T) {
 			Config:    plans.DynamicValue([]byte("config placeholder")),
 			Workspace: "default",
 		},
+		Checks: &states.CheckResults{},
 
 		// Due to some historical oddities in how we've changed modelling over
 		// time, we also include the states (without the corresponding file
@@ -84,11 +88,7 @@ func TestRoundtrip(t *testing.T) {
 		},
 	)
 
-	workDir, err := ioutil.TempDir("", "tf-planfile")
-	if err != nil {
-		t.Fatal(err)
-	}
-	planFn := filepath.Join(workDir, "tfplan")
+	planFn := filepath.Join(t.TempDir(), "tfplan")
 
 	err = Create(planFn, CreateArgs{
 		ConfigSnapshot:       snapIn,
@@ -101,10 +101,17 @@ func TestRoundtrip(t *testing.T) {
 		t.Fatalf("failed to create plan file: %s", err)
 	}
 
-	pr, err := Open(planFn)
+	wpf, err := OpenWrapped(planFn)
 	if err != nil {
 		t.Fatalf("failed to open plan file for reading: %s", err)
 	}
+	pr, ok := wpf.Local()
+	if !ok {
+		t.Fatalf("failed to open plan file as a local plan file")
+	}
+	if wpf.IsCloud() {
+		t.Fatalf("wrapped plan claims to be both kinds of plan at once")
+	}
 
 	t.Run("ReadPlan", func(t *testing.T) {
 		planOut, err := pr.ReadPlan()
@@ -168,3 +175,33 @@ func TestRoundtrip(t *testing.T) {
 		}
 	})
 }
+
+func TestWrappedError(t *testing.T) {
+	// Open something that isn't a cloud or local planfile: should error
+	wrongFile := "not a valid zip file"
+	_, err := OpenWrapped(filepath.Join("testdata", "test-config", "root.tf"))
+	if !strings.Contains(err.Error(), wrongFile) {
+		t.Fatalf("expected  %q, got %q", wrongFile, err)
+	}
+
+	// Open something that doesn't exist: should error
+	missingFile := "no such file or directory"
+	_, err = OpenWrapped(filepath.Join("testdata", "absent.tfplan"))
+	if !strings.Contains(err.Error(), missingFile) {
+		t.Fatalf("expected  %q, got %q", missingFile, err)
+	}
+}
+
+func TestWrappedCloud(t *testing.T) {
+	// Loading valid cloud plan results in a wrapped cloud plan
+	wpf, err := OpenWrapped(filepath.Join("testdata", "cloudplan.json"))
+	if err != nil {
+		t.Fatalf("failed to open valid cloud plan: %s", err)
+	}
+	if !wpf.IsCloud() {
+		t.Fatalf("failed to open cloud file as a cloud plan")
+	}
+	if wpf.IsLocal() {
+		t.Fatalf("wrapped plan claims to be both kinds of plan at once")
+	}
+}
