@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package refactoring
 
 import (
@@ -10,6 +13,7 @@ import (
 	"github.com/google/go-cmp/cmp"
 	"github.com/hashicorp/hcl/v2"
 	"github.com/hashicorp/hcl/v2/hclsyntax"
+
 	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/states"
 )
@@ -20,126 +24,15 @@ func TestApplyMoves(t *testing.T) {
 		Provider: addrs.MustParseProviderSourceString("example.com/foo/bar"),
 	}
 
-	moduleBoo, _ := addrs.ParseModuleInstanceStr("module.boo")
-	moduleBar, _ := addrs.ParseModuleInstanceStr("module.bar")
-	moduleBarKey, _ := addrs.ParseModuleInstanceStr("module.bar[0]")
-	moduleBooHoo, _ := addrs.ParseModuleInstanceStr("module.boo.module.hoo")
-	moduleBarHoo, _ := addrs.ParseModuleInstanceStr("module.bar.module.hoo")
-
-	instAddrs := map[string]addrs.AbsResourceInstance{
-		"foo.from": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "from",
-		}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance),
-
-		"foo.mid": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "mid",
-		}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance),
-
-		"foo.to": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "to",
-		}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance),
-
-		"foo.from[0]": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "from",
-		}.Instance(addrs.IntKey(0)).Absolute(addrs.RootModuleInstance),
-
-		"foo.to[0]": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "to",
-		}.Instance(addrs.IntKey(0)).Absolute(addrs.RootModuleInstance),
-
-		"module.boo.foo.from": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "from",
-		}.Instance(addrs.NoKey).Absolute(moduleBoo),
-
-		"module.boo.foo.mid": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "mid",
-		}.Instance(addrs.NoKey).Absolute(moduleBoo),
-
-		"module.boo.foo.to": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "to",
-		}.Instance(addrs.NoKey).Absolute(moduleBoo),
-
-		"module.boo.foo.from[0]": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "from",
-		}.Instance(addrs.IntKey(0)).Absolute(moduleBoo),
-
-		"module.boo.foo.to[0]": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "to",
-		}.Instance(addrs.IntKey(0)).Absolute(moduleBoo),
-
-		"module.bar.foo.from": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "from",
-		}.Instance(addrs.NoKey).Absolute(moduleBar),
-
-		"module.bar[0].foo.from": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "from",
-		}.Instance(addrs.NoKey).Absolute(moduleBarKey),
-
-		"module.bar[0].foo.mid": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "mid",
-		}.Instance(addrs.NoKey).Absolute(moduleBarKey),
-
-		"module.bar[0].foo.to": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "to",
-		}.Instance(addrs.NoKey).Absolute(moduleBarKey),
-
-		"module.bar[0].foo.from[0]": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "from",
-		}.Instance(addrs.IntKey(0)).Absolute(moduleBarKey),
-
-		"module.bar[0].foo.to[0]": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "to",
-		}.Instance(addrs.IntKey(0)).Absolute(moduleBarKey),
-
-		"module.boo.module.hoo.foo.from": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "from",
-		}.Instance(addrs.NoKey).Absolute(moduleBooHoo),
-
-		"module.bar.module.hoo.foo.from": addrs.Resource{
-			Mode: addrs.ManagedResourceMode,
-			Type: "foo",
-			Name: "from",
-		}.Instance(addrs.NoKey).Absolute(moduleBarHoo),
+	mustParseInstAddr := func(s string) addrs.AbsResourceInstance {
+		addr, err := addrs.ParseAbsResourceInstanceStr(s)
+		if err != nil {
+			t.Fatal(err)
+		}
+		return addr
 	}
 
-	emptyResults := MoveResults{
-		Changes: map[addrs.UniqueKey]MoveSuccess{},
-		Blocked: map[addrs.UniqueKey]MoveBlocked{},
-	}
+	emptyResults := makeMoveResults()
 
 	tests := map[string]struct {
 		Stmts []MoveStatement
@@ -158,7 +51,7 @@ func TestApplyMoves(t *testing.T) {
 			[]MoveStatement{},
 			states.BuildState(func(s *states.SyncState) {
 				s.SetResourceInstanceCurrent(
-					instAddrs["foo.from"],
+					mustParseInstAddr("foo.from"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -177,7 +70,7 @@ func TestApplyMoves(t *testing.T) {
 			},
 			states.BuildState(func(s *states.SyncState) {
 				s.SetResourceInstanceCurrent(
-					instAddrs["foo.from"],
+					mustParseInstAddr("foo.from"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -186,13 +79,13 @@ func TestApplyMoves(t *testing.T) {
 				)
 			}),
 			MoveResults{
-				Changes: map[addrs.UniqueKey]MoveSuccess{
-					instAddrs["foo.to"].UniqueKey(): {
-						From: instAddrs["foo.from"],
-						To:   instAddrs["foo.to"],
-					},
-				},
-				Blocked: map[addrs.UniqueKey]MoveBlocked{},
+				Changes: addrs.MakeMap(
+					addrs.MakeMapElem(mustParseInstAddr("foo.to"), MoveSuccess{
+						From: mustParseInstAddr("foo.from"),
+						To:   mustParseInstAddr("foo.to"),
+					}),
+				),
+				Blocked: emptyResults.Blocked,
 			},
 			[]string{
 				`foo.to`,
@@ -204,7 +97,7 @@ func TestApplyMoves(t *testing.T) {
 			},
 			states.BuildState(func(s *states.SyncState) {
 				s.SetResourceInstanceCurrent(
-					instAddrs["foo.from[0]"],
+					mustParseInstAddr("foo.from[0]"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -213,13 +106,13 @@ func TestApplyMoves(t *testing.T) {
 				)
 			}),
 			MoveResults{
-				Changes: map[addrs.UniqueKey]MoveSuccess{
-					instAddrs["foo.to[0]"].UniqueKey(): {
-						From: instAddrs["foo.from[0]"],
-						To:   instAddrs["foo.to[0]"],
-					},
-				},
-				Blocked: map[addrs.UniqueKey]MoveBlocked{},
+				Changes: addrs.MakeMap(
+					addrs.MakeMapElem(mustParseInstAddr("foo.to[0]"), MoveSuccess{
+						From: mustParseInstAddr("foo.from[0]"),
+						To:   mustParseInstAddr("foo.to[0]"),
+					}),
+				),
+				Blocked: emptyResults.Blocked,
 			},
 			[]string{
 				`foo.to[0]`,
@@ -232,7 +125,7 @@ func TestApplyMoves(t *testing.T) {
 			},
 			states.BuildState(func(s *states.SyncState) {
 				s.SetResourceInstanceCurrent(
-					instAddrs["foo.from"],
+					mustParseInstAddr("foo.from"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -241,13 +134,13 @@ func TestApplyMoves(t *testing.T) {
 				)
 			}),
 			MoveResults{
-				Changes: map[addrs.UniqueKey]MoveSuccess{
-					instAddrs["foo.to"].UniqueKey(): {
-						From: instAddrs["foo.from"],
-						To:   instAddrs["foo.to"],
-					},
-				},
-				Blocked: map[addrs.UniqueKey]MoveBlocked{},
+				Changes: addrs.MakeMap(
+					addrs.MakeMapElem(mustParseInstAddr("foo.to"), MoveSuccess{
+						From: mustParseInstAddr("foo.from"),
+						To:   mustParseInstAddr("foo.to"),
+					}),
+				),
+				Blocked: emptyResults.Blocked,
 			},
 			[]string{
 				`foo.to`,
@@ -260,7 +153,7 @@ func TestApplyMoves(t *testing.T) {
 			},
 			states.BuildState(func(s *states.SyncState) {
 				s.SetResourceInstanceCurrent(
-					instAddrs["foo.from[0]"],
+					mustParseInstAddr("foo.from[0]"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -269,13 +162,13 @@ func TestApplyMoves(t *testing.T) {
 				)
 			}),
 			MoveResults{
-				Changes: map[addrs.UniqueKey]MoveSuccess{
-					instAddrs["module.boo.foo.to[0]"].UniqueKey(): {
-						From: instAddrs["foo.from[0]"],
-						To:   instAddrs["module.boo.foo.to[0]"],
-					},
-				},
-				Blocked: map[addrs.UniqueKey]MoveBlocked{},
+				Changes: addrs.MakeMap(
+					addrs.MakeMapElem(mustParseInstAddr("module.boo.foo.to[0]"), MoveSuccess{
+						From: mustParseInstAddr("foo.from[0]"),
+						To:   mustParseInstAddr("module.boo.foo.to[0]"),
+					}),
+				),
+				Blocked: emptyResults.Blocked,
 			},
 			[]string{
 				`module.boo.foo.to[0]`,
@@ -288,7 +181,7 @@ func TestApplyMoves(t *testing.T) {
 			},
 			states.BuildState(func(s *states.SyncState) {
 				s.SetResourceInstanceCurrent(
-					instAddrs["module.boo.foo.from[0]"],
+					mustParseInstAddr("module.boo.foo.from[0]"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -297,13 +190,13 @@ func TestApplyMoves(t *testing.T) {
 				)
 			}),
 			MoveResults{
-				Changes: map[addrs.UniqueKey]MoveSuccess{
-					instAddrs["module.bar[0].foo.to[0]"].UniqueKey(): {
-						From: instAddrs["module.boo.foo.from[0]"],
-						To:   instAddrs["module.bar[0].foo.to[0]"],
-					},
-				},
-				Blocked: map[addrs.UniqueKey]MoveBlocked{},
+				Changes: addrs.MakeMap(
+					addrs.MakeMapElem(mustParseInstAddr("module.bar[0].foo.to[0]"), MoveSuccess{
+						From: mustParseInstAddr("module.boo.foo.from[0]"),
+						To:   mustParseInstAddr("module.bar[0].foo.to[0]"),
+					}),
+				),
+				Blocked: emptyResults.Blocked,
 			},
 			[]string{
 				`module.bar[0].foo.to[0]`,
@@ -316,7 +209,7 @@ func TestApplyMoves(t *testing.T) {
 			},
 			states.BuildState(func(s *states.SyncState) {
 				s.SetResourceInstanceCurrent(
-					instAddrs["module.boo.foo.from"],
+					mustParseInstAddr("module.boo.foo.from"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -324,7 +217,7 @@ func TestApplyMoves(t *testing.T) {
 					providerAddr,
 				)
 				s.SetResourceInstanceCurrent(
-					instAddrs["module.boo.module.hoo.foo.from"],
+					mustParseInstAddr("module.boo.module.hoo.foo.from"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -333,17 +226,17 @@ func TestApplyMoves(t *testing.T) {
 				)
 			}),
 			MoveResults{
-				Changes: map[addrs.UniqueKey]MoveSuccess{
-					instAddrs["module.bar.foo.from"].UniqueKey(): {
-						From: instAddrs["module.boo.foo.from"],
-						To:   instAddrs["module.bar.foo.from"],
-					},
-					instAddrs["module.bar.module.hoo.foo.from"].UniqueKey(): {
-						From: instAddrs["module.boo.module.hoo.foo.from"],
-						To:   instAddrs["module.bar.module.hoo.foo.from"],
-					},
-				},
-				Blocked: map[addrs.UniqueKey]MoveBlocked{},
+				Changes: addrs.MakeMap(
+					addrs.MakeMapElem(mustParseInstAddr("module.bar.foo.from"), MoveSuccess{
+						From: mustParseInstAddr("module.boo.foo.from"),
+						To:   mustParseInstAddr("module.bar.foo.from"),
+					}),
+					addrs.MakeMapElem(mustParseInstAddr("module.bar.module.hoo.foo.from"), MoveSuccess{
+						From: mustParseInstAddr("module.boo.module.hoo.foo.from"),
+						To:   mustParseInstAddr("module.bar.module.hoo.foo.from"),
+					}),
+				),
+				Blocked: emptyResults.Blocked,
 			},
 			[]string{
 				`module.bar.foo.from`,
@@ -357,7 +250,7 @@ func TestApplyMoves(t *testing.T) {
 			},
 			states.BuildState(func(s *states.SyncState) {
 				s.SetResourceInstanceCurrent(
-					instAddrs["module.boo.foo.from[0]"],
+					mustParseInstAddr("module.boo.foo.from[0]"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -366,13 +259,13 @@ func TestApplyMoves(t *testing.T) {
 				)
 			}),
 			MoveResults{
-				Changes: map[addrs.UniqueKey]MoveSuccess{
-					instAddrs["module.bar[0].foo.from[0]"].UniqueKey(): {
-						From: instAddrs["module.boo.foo.from[0]"],
-						To:   instAddrs["module.bar[0].foo.from[0]"],
-					},
-				},
-				Blocked: map[addrs.UniqueKey]MoveBlocked{},
+				Changes: addrs.MakeMap(
+					addrs.MakeMapElem(mustParseInstAddr("module.bar[0].foo.from[0]"), MoveSuccess{
+						From: mustParseInstAddr("module.boo.foo.from[0]"),
+						To:   mustParseInstAddr("module.bar[0].foo.from[0]"),
+					}),
+				),
+				Blocked: emptyResults.Blocked,
 			},
 			[]string{
 				`module.bar[0].foo.from[0]`,
@@ -386,7 +279,7 @@ func TestApplyMoves(t *testing.T) {
 			},
 			states.BuildState(func(s *states.SyncState) {
 				s.SetResourceInstanceCurrent(
-					instAddrs["module.boo.foo.from[0]"],
+					mustParseInstAddr("module.boo.foo.from[0]"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -395,13 +288,13 @@ func TestApplyMoves(t *testing.T) {
 				)
 			}),
 			MoveResults{
-				Changes: map[addrs.UniqueKey]MoveSuccess{
-					instAddrs["module.bar[0].foo.to[0]"].UniqueKey(): {
-						From: instAddrs["module.boo.foo.from[0]"],
-						To:   instAddrs["module.bar[0].foo.to[0]"],
-					},
-				},
-				Blocked: map[addrs.UniqueKey]MoveBlocked{},
+				Changes: addrs.MakeMap(
+					addrs.MakeMapElem(mustParseInstAddr("module.bar[0].foo.to[0]"), MoveSuccess{
+						From: mustParseInstAddr("module.boo.foo.from[0]"),
+						To:   mustParseInstAddr("module.bar[0].foo.to[0]"),
+					}),
+				),
+				Blocked: emptyResults.Blocked,
 			},
 			[]string{
 				`module.bar[0].foo.to[0]`,
@@ -415,7 +308,7 @@ func TestApplyMoves(t *testing.T) {
 			},
 			states.BuildState(func(s *states.SyncState) {
 				s.SetResourceInstanceCurrent(
-					instAddrs["module.boo.foo.from[0]"],
+					mustParseInstAddr("module.boo.foo.from[0]"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -424,13 +317,13 @@ func TestApplyMoves(t *testing.T) {
 				)
 			}),
 			MoveResults{
-				Changes: map[addrs.UniqueKey]MoveSuccess{
-					instAddrs["module.bar[0].foo.to[0]"].UniqueKey(): {
-						From: instAddrs["module.boo.foo.from[0]"],
-						To:   instAddrs["module.bar[0].foo.to[0]"],
-					},
-				},
-				Blocked: map[addrs.UniqueKey]MoveBlocked{},
+				Changes: addrs.MakeMap(
+					addrs.MakeMapElem(mustParseInstAddr("module.bar[0].foo.to[0]"), MoveSuccess{
+						From: mustParseInstAddr("module.boo.foo.from[0]"),
+						To:   mustParseInstAddr("module.bar[0].foo.to[0]"),
+					}),
+				),
+				Blocked: emptyResults.Blocked,
 			},
 			[]string{
 				`module.bar[0].foo.to[0]`,
@@ -443,7 +336,7 @@ func TestApplyMoves(t *testing.T) {
 			},
 			states.BuildState(func(s *states.SyncState) {
 				s.SetResourceInstanceCurrent(
-					instAddrs["module.bar[0].foo.from"],
+					mustParseInstAddr("module.bar[0].foo.from"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -451,7 +344,7 @@ func TestApplyMoves(t *testing.T) {
 					providerAddr,
 				)
 				s.SetResourceInstanceCurrent(
-					instAddrs["module.boo.foo.to[0]"],
+					mustParseInstAddr("module.boo.foo.to[0]"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -462,13 +355,16 @@ func TestApplyMoves(t *testing.T) {
 			MoveResults{
 				// Nothing moved, because the module.b address is already
 				// occupied by another module.
-				Changes: map[addrs.UniqueKey]MoveSuccess{},
-				Blocked: map[addrs.UniqueKey]MoveBlocked{
-					instAddrs["module.bar[0].foo.from"].Module.UniqueKey(): {
-						Wanted: instAddrs["module.boo.foo.to[0]"].Module,
-						Actual: instAddrs["module.bar[0].foo.from"].Module,
-					},
-				},
+				Changes: emptyResults.Changes,
+				Blocked: addrs.MakeMap(
+					addrs.MakeMapElem[addrs.AbsMoveable](
+						mustParseInstAddr("module.bar[0].foo.from").Module,
+						MoveBlocked{
+							Wanted: mustParseInstAddr("module.boo.foo.to[0]").Module,
+							Actual: mustParseInstAddr("module.bar[0].foo.from").Module,
+						},
+					),
+				),
 			},
 			[]string{
 				`module.bar[0].foo.from`,
@@ -482,7 +378,7 @@ func TestApplyMoves(t *testing.T) {
 			},
 			states.BuildState(func(s *states.SyncState) {
 				s.SetResourceInstanceCurrent(
-					instAddrs["foo.from"],
+					mustParseInstAddr("foo.from"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -490,7 +386,7 @@ func TestApplyMoves(t *testing.T) {
 					providerAddr,
 				)
 				s.SetResourceInstanceCurrent(
-					instAddrs["foo.to"],
+					mustParseInstAddr("foo.to"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -501,13 +397,16 @@ func TestApplyMoves(t *testing.T) {
 			MoveResults{
 				// Nothing moved, because the from.to address is already
 				// occupied by another resource.
-				Changes: map[addrs.UniqueKey]MoveSuccess{},
-				Blocked: map[addrs.UniqueKey]MoveBlocked{
-					instAddrs["foo.from"].ContainingResource().UniqueKey(): {
-						Wanted: instAddrs["foo.to"].ContainingResource(),
-						Actual: instAddrs["foo.from"].ContainingResource(),
-					},
-				},
+				Changes: emptyResults.Changes,
+				Blocked: addrs.MakeMap(
+					addrs.MakeMapElem[addrs.AbsMoveable](
+						mustParseInstAddr("foo.from").ContainingResource(),
+						MoveBlocked{
+							Wanted: mustParseInstAddr("foo.to").ContainingResource(),
+							Actual: mustParseInstAddr("foo.from").ContainingResource(),
+						},
+					),
+				),
 			},
 			[]string{
 				`foo.from`,
@@ -521,7 +420,7 @@ func TestApplyMoves(t *testing.T) {
 			},
 			states.BuildState(func(s *states.SyncState) {
 				s.SetResourceInstanceCurrent(
-					instAddrs["foo.from"],
+					mustParseInstAddr("foo.from"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -529,7 +428,7 @@ func TestApplyMoves(t *testing.T) {
 					providerAddr,
 				)
 				s.SetResourceInstanceCurrent(
-					instAddrs["foo.to[0]"],
+					mustParseInstAddr("foo.to[0]"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -540,13 +439,16 @@ func TestApplyMoves(t *testing.T) {
 			MoveResults{
 				// Nothing moved, because the from.to[0] address is already
 				// occupied by another resource instance.
-				Changes: map[addrs.UniqueKey]MoveSuccess{},
-				Blocked: map[addrs.UniqueKey]MoveBlocked{
-					instAddrs["foo.from"].UniqueKey(): {
-						Wanted: instAddrs["foo.to[0]"],
-						Actual: instAddrs["foo.from"],
-					},
-				},
+				Changes: emptyResults.Changes,
+				Blocked: addrs.MakeMap(
+					addrs.MakeMapElem[addrs.AbsMoveable](
+						mustParseInstAddr("foo.from"),
+						MoveBlocked{
+							Wanted: mustParseInstAddr("foo.to[0]"),
+							Actual: mustParseInstAddr("foo.from"),
+						},
+					),
+				),
 			},
 			[]string{
 				`foo.from`,
@@ -560,7 +462,7 @@ func TestApplyMoves(t *testing.T) {
 			},
 			states.BuildState(func(s *states.SyncState) {
 				s.SetResourceInstanceCurrent(
-					instAddrs["module.boo.foo.from"],
+					mustParseInstAddr("module.boo.foo.from"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -569,13 +471,13 @@ func TestApplyMoves(t *testing.T) {
 				)
 			}),
 			MoveResults{
-				Changes: map[addrs.UniqueKey]MoveSuccess{
-					instAddrs["module.bar[0].foo.to"].UniqueKey(): {
-						From: instAddrs["module.boo.foo.from"],
-						To:   instAddrs["module.bar[0].foo.to"],
-					},
-				},
-				Blocked: map[addrs.UniqueKey]MoveBlocked{},
+				Changes: addrs.MakeMap(
+					addrs.MakeMapElem(mustParseInstAddr("module.bar[0].foo.to"), MoveSuccess{
+						From: mustParseInstAddr("module.boo.foo.from"),
+						To:   mustParseInstAddr("module.bar[0].foo.to"),
+					}),
+				),
+				Blocked: emptyResults.Blocked,
 			},
 			[]string{
 				`module.bar[0].foo.to`,
@@ -589,7 +491,7 @@ func TestApplyMoves(t *testing.T) {
 			},
 			states.BuildState(func(s *states.SyncState) {
 				s.SetResourceInstanceCurrent(
-					instAddrs["module.bar[0].foo.to"],
+					mustParseInstAddr("module.bar[0].foo.to"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -597,7 +499,7 @@ func TestApplyMoves(t *testing.T) {
 					providerAddr,
 				)
 				s.SetResourceInstanceCurrent(
-					instAddrs["foo.from"],
+					mustParseInstAddr("foo.from"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -606,17 +508,17 @@ func TestApplyMoves(t *testing.T) {
 				)
 			}),
 			MoveResults{
-				Changes: map[addrs.UniqueKey]MoveSuccess{
-					instAddrs["module.boo.foo.from"].UniqueKey(): {
-						instAddrs["foo.from"],
-						instAddrs["module.boo.foo.from"],
-					},
-					instAddrs["module.boo.foo.to"].UniqueKey(): {
-						instAddrs["module.bar[0].foo.to"],
-						instAddrs["module.boo.foo.to"],
-					},
-				},
-				Blocked: map[addrs.UniqueKey]MoveBlocked{},
+				Changes: addrs.MakeMap(
+					addrs.MakeMapElem(mustParseInstAddr("module.boo.foo.from"), MoveSuccess{
+						mustParseInstAddr("foo.from"),
+						mustParseInstAddr("module.boo.foo.from"),
+					}),
+					addrs.MakeMapElem(mustParseInstAddr("module.boo.foo.to"), MoveSuccess{
+						mustParseInstAddr("module.bar[0].foo.to"),
+						mustParseInstAddr("module.boo.foo.to"),
+					}),
+				),
+				Blocked: emptyResults.Blocked,
 			},
 			[]string{
 				`module.boo.foo.from`,
@@ -624,6 +526,49 @@ func TestApplyMoves(t *testing.T) {
 			},
 		},
 
+		"move resources into module and then move module": {
+			[]MoveStatement{
+				testMoveStatement(t, "", "foo.from", "module.boo.foo.to"),
+				testMoveStatement(t, "", "bar.from", "module.boo.bar.to"),
+				testMoveStatement(t, "", "module.boo", "module.bar[0]"),
+			},
+			states.BuildState(func(s *states.SyncState) {
+				s.SetResourceInstanceCurrent(
+					mustParseInstAddr("foo.from"),
+					&states.ResourceInstanceObjectSrc{
+						Status:    states.ObjectReady,
+						AttrsJSON: []byte(`{}`),
+					},
+					providerAddr,
+				)
+				s.SetResourceInstanceCurrent(
+					mustParseInstAddr("bar.from"),
+					&states.ResourceInstanceObjectSrc{
+						Status:    states.ObjectReady,
+						AttrsJSON: []byte(`{}`),
+					},
+					providerAddr,
+				)
+			}),
+			MoveResults{
+				Changes: addrs.MakeMap(
+					addrs.MakeMapElem(mustParseInstAddr("module.bar[0].foo.to"), MoveSuccess{
+						mustParseInstAddr("foo.from"),
+						mustParseInstAddr("module.bar[0].foo.to"),
+					}),
+					addrs.MakeMapElem(mustParseInstAddr("module.bar[0].bar.to"), MoveSuccess{
+						mustParseInstAddr("bar.from"),
+						mustParseInstAddr("module.bar[0].bar.to"),
+					}),
+				),
+				Blocked: emptyResults.Blocked,
+			},
+			[]string{
+				`module.bar[0].bar.to`,
+				`module.bar[0].foo.to`,
+			},
+		},
+
 		"module move collides with resource move": {
 			[]MoveStatement{
 				testMoveStatement(t, "", "module.bar[0]", "module.boo"),
@@ -631,7 +576,7 @@ func TestApplyMoves(t *testing.T) {
 			},
 			states.BuildState(func(s *states.SyncState) {
 				s.SetResourceInstanceCurrent(
-					instAddrs["module.bar[0].foo.from"],
+					mustParseInstAddr("module.bar[0].foo.from"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -639,7 +584,7 @@ func TestApplyMoves(t *testing.T) {
 					providerAddr,
 				)
 				s.SetResourceInstanceCurrent(
-					instAddrs["foo.from"],
+					mustParseInstAddr("foo.from"),
 					&states.ResourceInstanceObjectSrc{
 						Status:    states.ObjectReady,
 						AttrsJSON: []byte(`{}`),
@@ -648,19 +593,21 @@ func TestApplyMoves(t *testing.T) {
 				)
 			}),
 			MoveResults{
-				Changes: map[addrs.UniqueKey]MoveSuccess{
-
-					instAddrs["module.boo.foo.from"].UniqueKey(): {
-						instAddrs["module.bar[0].foo.from"],
-						instAddrs["module.boo.foo.from"],
-					},
-				},
-				Blocked: map[addrs.UniqueKey]MoveBlocked{
-					instAddrs["foo.from"].ContainingResource().UniqueKey(): {
-						Actual: instAddrs["foo.from"].ContainingResource(),
-						Wanted: instAddrs["module.boo.foo.from"].ContainingResource(),
-					},
-				},
+				Changes: addrs.MakeMap(
+					addrs.MakeMapElem(mustParseInstAddr("module.boo.foo.from"), MoveSuccess{
+						mustParseInstAddr("module.bar[0].foo.from"),
+						mustParseInstAddr("module.boo.foo.from"),
+					}),
+				),
+				Blocked: addrs.MakeMap(
+					addrs.MakeMapElem[addrs.AbsMoveable](
+						mustParseInstAddr("foo.from").ContainingResource(),
+						MoveBlocked{
+							Actual: mustParseInstAddr("foo.from").ContainingResource(),
+							Wanted: mustParseInstAddr("module.boo.foo.from").ContainingResource(),
+						},
+					),
+				),
 			},
 			[]string{
 				`foo.from`,
