@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package remote
 
 import (
@@ -6,8 +9,11 @@ import (
 	"testing"
 
 	"github.com/google/go-cmp/cmp"
+	"github.com/google/go-cmp/cmp/cmpopts"
 	"github.com/zclconf/go-cty/cty"
 
+	tfaddr "github.com/hashicorp/terraform-registry-address"
+	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/states"
 	"github.com/hashicorp/terraform/internal/states/statefile"
 	"github.com/hashicorp/terraform/internal/states/statemgr"
@@ -19,6 +25,7 @@ func TestState_impl(t *testing.T) {
 	var _ statemgr.Writer = new(State)
 	var _ statemgr.Persister = new(State)
 	var _ statemgr.Refresher = new(State)
+	var _ statemgr.OutputReader = new(State)
 	var _ statemgr.Locker = new(State)
 }
 
@@ -36,7 +43,7 @@ func TestStateRace(t *testing.T) {
 		go func() {
 			defer wg.Done()
 			s.WriteState(current)
-			s.PersistState()
+			s.PersistState(nil)
 			s.RefreshState()
 		}()
 	}
@@ -48,8 +55,8 @@ type testCase struct {
 	name string
 	// A function to mutate state and return a cleanup function
 	mutationFunc func(*State) (*states.State, func())
-	// The expected request to have taken place
-	expectedRequest mockClientRequest
+	// The expected requests to have taken place
+	expectedRequests []mockClientRequest
 	// Mark this case as not having a request
 	noRequest bool
 }
@@ -58,54 +65,145 @@ type testCase struct {
 // a request doesn't have one by checking if a method exists
 // on the expectedRequest.
 func (tc testCase) isRequested(t *testing.T) bool {
-	hasMethod := tc.expectedRequest.Method != ""
-	if tc.noRequest && hasMethod {
-		t.Fatalf("expected no content for %q but got: %v", tc.name, tc.expectedRequest)
+	for _, expectedMethod := range tc.expectedRequests {
+		hasMethod := expectedMethod.Method != ""
+		if tc.noRequest && hasMethod {
+			t.Fatalf("expected no content for %q but got: %v", tc.name, expectedMethod)
+		}
 	}
 	return !tc.noRequest
 }
 
 func TestStatePersist(t *testing.T) {
 	testCases := []testCase{
-		// Refreshing state before we run the test loop causes a GET
 		{
-			name: "refresh state",
+			name: "first state persistence",
 			mutationFunc: func(mgr *State) (*states.State, func()) {
-				return mgr.State(), func() {}
+				mgr.state = &states.State{
+					Modules: map[string]*states.Module{"": {}},
+				}
+				s := mgr.State()
+				s.RootModule().SetResourceInstanceCurrent(
+					addrs.Resource{
+						Mode: addrs.ManagedResourceMode,
+						Name: "myfile",
+						Type: "local_file",
+					}.Instance(addrs.NoKey),
+					&states.ResourceInstanceObjectSrc{
+						AttrsFlat: map[string]string{
+							"filename": "file.txt",
+						},
+						Status: states.ObjectReady,
+					},
+					addrs.AbsProviderConfig{
+						Provider: tfaddr.Provider{Namespace: "local"},
+					},
+				)
+				return s, func() {}
 			},
-			expectedRequest: mockClientRequest{
-				Method: "Get",
-				Content: map[string]interface{}{
-					"version":           4.0, // encoding/json decodes this as float64 by default
-					"lineage":           "mock-lineage",
-					"serial":            1.0, // encoding/json decodes this as float64 by default
-					"terraform_version": "0.0.0",
-					"outputs":           map[string]interface{}{},
-					"resources":         []interface{}{},
+			expectedRequests: []mockClientRequest{
+				// Expect an initial refresh, which returns nothing since there is no remote state.
+				{
+					Method:  "Get",
+					Content: nil,
+				},
+				// Expect a second refresh, since the read state is nil
+				{
+					Method:  "Get",
+					Content: nil,
+				},
+				// Expect an initial push with values and a serial of 1
+				{
+					Method: "Put",
+					Content: map[string]interface{}{
+						"version":           4.0, // encoding/json decodes this as float64 by default
+						"lineage":           "some meaningless value",
+						"serial":            1.0, // encoding/json decodes this as float64 by default
+						"terraform_version": version.Version,
+						"outputs":           map[string]interface{}{},
+						"resources": []interface{}{
+							map[string]interface{}{
+								"instances": []interface{}{
+									map[string]interface{}{
+										"attributes_flat": map[string]interface{}{
+											"filename": "file.txt",
+										},
+										"schema_version":       0.0,
+										"sensitive_attributes": []interface{}{},
+									},
+								},
+								"mode":     "managed",
+								"name":     "myfile",
+								"provider": `provider["/local/"]`,
+								"type":     "local_file",
+							},
+						},
+						"check_results": nil,
+					},
 				},
 			},
 		},
+		// If lineage changes, expect the serial to increment
 		{
 			name: "change lineage",
 			mutationFunc: func(mgr *State) (*states.State, func()) {
-				originalLineage := mgr.lineage
-				mgr.lineage = "some-new-lineage"
-				return mgr.State(), func() {
-					mgr.lineage = originalLineage
-				}
+				mgr.lineage = "mock-lineage"
+				return mgr.State(), func() {}
 			},
-			expectedRequest: mockClientRequest{
-				Method: "Put",
-				Content: map[string]interface{}{
-					"version":           4.0, // encoding/json decodes this as float64 by default
-					"lineage":           "some-new-lineage",
-					"serial":            2.0, // encoding/json decodes this as float64 by default
-					"terraform_version": version.Version,
-					"outputs":           map[string]interface{}{},
-					"resources":         []interface{}{},
+			expectedRequests: []mockClientRequest{
+				{
+					Method: "Put",
+					Content: map[string]interface{}{
+						"version":           4.0, // encoding/json decodes this as float64 by default
+						"lineage":           "mock-lineage",
+						"serial":            2.0, // encoding/json decodes this as float64 by default
+						"terraform_version": version.Version,
+						"outputs":           map[string]interface{}{},
+						"resources": []interface{}{
+							map[string]interface{}{
+								"instances": []interface{}{
+									map[string]interface{}{
+										"attributes_flat": map[string]interface{}{
+											"filename": "file.txt",
+										},
+										"schema_version":       0.0,
+										"sensitive_attributes": []interface{}{},
+									},
+								},
+								"mode":     "managed",
+								"name":     "myfile",
+								"provider": `provider["/local/"]`,
+								"type":     "local_file",
+							},
+						},
+						"check_results": nil,
+					},
 				},
 			},
 		},
+		// removing resources should increment the serial
+		{
+			name: "remove resources",
+			mutationFunc: func(mgr *State) (*states.State, func()) {
+				mgr.state.RootModule().Resources = map[string]*states.Resource{}
+				return mgr.State(), func() {}
+			},
+			expectedRequests: []mockClientRequest{
+				{
+					Method: "Put",
+					Content: map[string]interface{}{
+						"version":           4.0, // encoding/json decodes this as float64 by default
+						"lineage":           "mock-lineage",
+						"serial":            3.0, // encoding/json decodes this as float64 by default
+						"terraform_version": version.Version,
+						"outputs":           map[string]interface{}{},
+						"resources":         []interface{}{},
+						"check_results":     nil,
+					},
+				},
+			},
+		},
+		// If the remote serial is incremented, then we increment it once more.
 		{
 			name: "change serial",
 			mutationFunc: func(mgr *State) (*states.State, func()) {
@@ -115,18 +213,22 @@ func TestStatePersist(t *testing.T) {
 					mgr.serial = originalSerial
 				}
 			},
-			expectedRequest: mockClientRequest{
-				Method: "Put",
-				Content: map[string]interface{}{
-					"version":           4.0, // encoding/json decodes this as float64 by default
-					"lineage":           "mock-lineage",
-					"serial":            4.0, // encoding/json decodes this as float64 by default
-					"terraform_version": version.Version,
-					"outputs":           map[string]interface{}{},
-					"resources":         []interface{}{},
+			expectedRequests: []mockClientRequest{
+				{
+					Method: "Put",
+					Content: map[string]interface{}{
+						"version":           4.0, // encoding/json decodes this as float64 by default
+						"lineage":           "mock-lineage",
+						"serial":            5.0, // encoding/json decodes this as float64 by default
+						"terraform_version": version.Version,
+						"outputs":           map[string]interface{}{},
+						"resources":         []interface{}{},
+						"check_results":     nil,
+					},
 				},
 			},
 		},
+		// Adding an output should cause the serial to increment as well.
 		{
 			name: "add output to state",
 			mutationFunc: func(mgr *State) (*states.State, func()) {
@@ -134,23 +236,27 @@ func TestStatePersist(t *testing.T) {
 				s.RootModule().SetOutputValue("foo", cty.StringVal("bar"), false)
 				return s, func() {}
 			},
-			expectedRequest: mockClientRequest{
-				Method: "Put",
-				Content: map[string]interface{}{
-					"version":           4.0, // encoding/json decodes this as float64 by default
-					"lineage":           "mock-lineage",
-					"serial":            3.0, // encoding/json decodes this as float64 by default
-					"terraform_version": version.Version,
-					"outputs": map[string]interface{}{
-						"foo": map[string]interface{}{
-							"type":  "string",
-							"value": "bar",
+			expectedRequests: []mockClientRequest{
+				{
+					Method: "Put",
+					Content: map[string]interface{}{
+						"version":           4.0, // encoding/json decodes this as float64 by default
+						"lineage":           "mock-lineage",
+						"serial":            4.0, // encoding/json decodes this as float64 by default
+						"terraform_version": version.Version,
+						"outputs": map[string]interface{}{
+							"foo": map[string]interface{}{
+								"type":  "string",
+								"value": "bar",
+							},
 						},
+						"resources":     []interface{}{},
+						"check_results": nil,
 					},
-					"resources": []interface{}{},
 				},
 			},
 		},
+		// ...as should changing an output
 		{
 			name: "mutate state bar -> baz",
 			mutationFunc: func(mgr *State) (*states.State, func()) {
@@ -158,20 +264,23 @@ func TestStatePersist(t *testing.T) {
 				s.RootModule().SetOutputValue("foo", cty.StringVal("baz"), false)
 				return s, func() {}
 			},
-			expectedRequest: mockClientRequest{
-				Method: "Put",
-				Content: map[string]interface{}{
-					"version":           4.0, // encoding/json decodes this as float64 by default
-					"lineage":           "mock-lineage",
-					"serial":            4.0, // encoding/json decodes this as float64 by default
-					"terraform_version": version.Version,
-					"outputs": map[string]interface{}{
-						"foo": map[string]interface{}{
-							"type":  "string",
-							"value": "baz",
+			expectedRequests: []mockClientRequest{
+				{
+					Method: "Put",
+					Content: map[string]interface{}{
+						"version":           4.0, // encoding/json decodes this as float64 by default
+						"lineage":           "mock-lineage",
+						"serial":            5.0, // encoding/json decodes this as float64 by default
+						"terraform_version": version.Version,
+						"outputs": map[string]interface{}{
+							"foo": map[string]interface{}{
+								"type":  "string",
+								"value": "baz",
+							},
 						},
+						"resources":     []interface{}{},
+						"check_results": nil,
 					},
-					"resources": []interface{}{},
 				},
 			},
 		},
@@ -183,26 +292,31 @@ func TestStatePersist(t *testing.T) {
 			},
 			noRequest: true,
 		},
+		// If the remote state's serial is less (force push), then we
+		// increment it once from there.
 		{
 			name: "reset serial (force push style)",
 			mutationFunc: func(mgr *State) (*states.State, func()) {
 				mgr.serial = 2
 				return mgr.State(), func() {}
 			},
-			expectedRequest: mockClientRequest{
-				Method: "Put",
-				Content: map[string]interface{}{
-					"version":           4.0, // encoding/json decodes this as float64 by default
-					"lineage":           "mock-lineage",
-					"serial":            3.0, // encoding/json decodes this as float64 by default
-					"terraform_version": version.Version,
-					"outputs": map[string]interface{}{
-						"foo": map[string]interface{}{
-							"type":  "string",
-							"value": "baz",
+			expectedRequests: []mockClientRequest{
+				{
+					Method: "Put",
+					Content: map[string]interface{}{
+						"version":           4.0, // encoding/json decodes this as float64 by default
+						"lineage":           "mock-lineage",
+						"serial":            3.0, // encoding/json decodes this as float64 by default
+						"terraform_version": version.Version,
+						"outputs": map[string]interface{}{
+							"foo": map[string]interface{}{
+								"type":  "string",
+								"value": "baz",
+							},
 						},
+						"resources":     []interface{}{},
+						"check_results": nil,
 					},
-					"resources": []interface{}{},
 				},
 			},
 		},
@@ -212,18 +326,7 @@ func TestStatePersist(t *testing.T) {
 	// test assertions below, or else we'd need to deal with
 	// random lineage.
 	mgr := &State{
-		Client: &mockClient{
-			current: []byte(`
-				{
-					"version": 4,
-					"lineage": "mock-lineage",
-					"serial": 1,
-					"terraform_version":"0.0.0",
-					"outputs": {},
-					"resources": []
-				}
-			`),
-		},
+		Client: &mockClient{},
 	}
 
 	// In normal use (during a Terraform operation) we always refresh and read
@@ -245,32 +348,67 @@ func TestStatePersist(t *testing.T) {
 
 	// Run tests in order.
 	for _, tc := range testCases {
-		s, cleanup := tc.mutationFunc(mgr)
+		t.Run(tc.name, func(t *testing.T) {
+			s, cleanup := tc.mutationFunc(mgr)
 
-		if err := mgr.WriteState(s); err != nil {
-			t.Fatalf("failed to WriteState for %q: %s", tc.name, err)
-		}
-		if err := mgr.PersistState(); err != nil {
-			t.Fatalf("failed to PersistState for %q: %s", tc.name, err)
-		}
+			if err := mgr.WriteState(s); err != nil {
+				t.Fatalf("failed to WriteState for %q: %s", tc.name, err)
+			}
+			if err := mgr.PersistState(nil); err != nil {
+				t.Fatalf("failed to PersistState for %q: %s", tc.name, err)
+			}
 
-		if tc.isRequested(t) {
-			// Get captured request from the mock client log
-			// based on the index of the current test
-			if logIdx >= len(mockClient.log) {
-				t.Fatalf("request lock and index are out of sync on %q: idx=%d len=%d", tc.name, logIdx, len(mockClient.log))
+			if tc.isRequested(t) {
+				// Get captured request from the mock client log
+				// based on the index of the current test
+				if logIdx >= len(mockClient.log) {
+					t.Fatalf("request lock and index are out of sync on %q: idx=%d len=%d", tc.name, logIdx, len(mockClient.log))
+				}
+				for expectedRequestIdx := 0; expectedRequestIdx < len(tc.expectedRequests); expectedRequestIdx++ {
+					loggedRequest := mockClient.log[logIdx]
+					logIdx++
+					if diff := cmp.Diff(tc.expectedRequests[expectedRequestIdx], loggedRequest, cmpopts.IgnoreMapEntries(func(key string, value interface{}) bool {
+						// This is required since the initial state creation causes the lineage to be a UUID that is not known at test time.
+						return tc.name == "first state persistence" && key == "lineage"
+					})); len(diff) > 0 {
+						t.Logf("incorrect client requests for %q:\n%s", tc.name, diff)
+						t.Fail()
+					}
+				}
 			}
-			loggedRequest := mockClient.log[logIdx]
-			logIdx++
-			if diff := cmp.Diff(tc.expectedRequest, loggedRequest); len(diff) > 0 {
-				t.Fatalf("incorrect client requests for %q:\n%s", tc.name, diff)
-			}
-		}
-		cleanup()
+			cleanup()
+		})
 	}
 	logCnt := len(mockClient.log)
 	if logIdx != logCnt {
-		log.Fatalf("not all requests were read. Expected logIdx to be %d but got %d", logCnt, logIdx)
+		t.Fatalf("not all requests were read. Expected logIdx to be %d but got %d", logCnt, logIdx)
+	}
+}
+
+func TestState_GetRootOutputValues(t *testing.T) {
+	// Initial setup of state with outputs already defined
+	mgr := &State{
+		Client: &mockClient{
+			current: []byte(`
+				{
+					"version": 4,
+					"lineage": "mock-lineage",
+					"serial": 1,
+					"terraform_version":"0.0.0",
+					"outputs": {"foo": {"value":"bar", "type": "string"}},
+					"resources": []
+				}
+			`),
+		},
+	}
+
+	outputs, err := mgr.GetRootOutputValues()
+	if err != nil {
+		t.Errorf("Expected GetRootOutputValues to not return an error, but it returned %v", err)
+	}
+
+	if len(outputs) != 1 {
+		t.Errorf("Expected %d outputs, but received %d", 1, len(outputs))
 	}
 }
 
@@ -349,6 +487,7 @@ func TestWriteStateForMigration(t *testing.T) {
 					"terraform_version": version.Version,
 					"outputs":           map[string]interface{}{"foo": map[string]interface{}{"type": string("string"), "value": string("bar")}},
 					"resources":         []interface{}{},
+					"check_results":     nil,
 				},
 			},
 			force: true,
@@ -367,6 +506,7 @@ func TestWriteStateForMigration(t *testing.T) {
 					"terraform_version": version.Version,
 					"outputs":           map[string]interface{}{"foo": map[string]interface{}{"type": string("string"), "value": string("bar")}},
 					"resources":         []interface{}{},
+					"check_results":     nil,
 				},
 			},
 			force: true,
@@ -395,37 +535,39 @@ func TestWriteStateForMigration(t *testing.T) {
 	logIdx := 0
 
 	for _, tc := range testCases {
-		sf := tc.stateFile(mgr)
-		err := mgr.WriteStateForMigration(sf, tc.force)
-		shouldError := tc.expectedError != ""
+		t.Run(tc.name, func(t *testing.T) {
+			sf := tc.stateFile(mgr)
+			err := mgr.WriteStateForMigration(sf, tc.force)
+			shouldError := tc.expectedError != ""
 
-		// If we are expecting and error check it and move on
-		if shouldError {
-			if err == nil {
-				t.Fatalf("test case %q should have failed with error %q", tc.name, tc.expectedError)
-			} else if err.Error() != tc.expectedError {
-				t.Fatalf("test case %q expected error %q but got %q", tc.name, tc.expectedError, err)
+			// If we are expecting and error check it and move on
+			if shouldError {
+				if err == nil {
+					t.Fatalf("test case %q should have failed with error %q", tc.name, tc.expectedError)
+				} else if err.Error() != tc.expectedError {
+					t.Fatalf("test case %q expected error %q but got %q", tc.name, tc.expectedError, err)
+				}
+				return
 			}
-			continue
-		}
 
-		if err != nil {
-			t.Fatalf("test case %q failed: %v", tc.name, err)
-		}
+			if err != nil {
+				t.Fatalf("test case %q failed: %v", tc.name, err)
+			}
 
-		// At this point we should just do a normal write and persist
-		// as would happen from the CLI
-		mgr.WriteState(mgr.State())
-		mgr.PersistState()
+			// At this point we should just do a normal write and persist
+			// as would happen from the CLI
+			mgr.WriteState(mgr.State())
+			mgr.PersistState(nil)
 
-		if logIdx >= len(mockClient.log) {
-			t.Fatalf("request lock and index are out of sync on %q: idx=%d len=%d", tc.name, logIdx, len(mockClient.log))
-		}
-		loggedRequest := mockClient.log[logIdx]
-		logIdx++
-		if diff := cmp.Diff(tc.expectedRequest, loggedRequest); len(diff) > 0 {
-			t.Fatalf("incorrect client requests for %q:\n%s", tc.name, diff)
-		}
+			if logIdx >= len(mockClient.log) {
+				t.Fatalf("request lock and index are out of sync on %q: idx=%d len=%d", tc.name, logIdx, len(mockClient.log))
+			}
+			loggedRequest := mockClient.log[logIdx]
+			logIdx++
+			if diff := cmp.Diff(tc.expectedRequest, loggedRequest); len(diff) > 0 {
+				t.Fatalf("incorrect client requests for %q:\n%s", tc.name, diff)
+			}
+		})
 	}
 
 	logCnt := len(mockClient.log)
@@ -501,6 +643,7 @@ func TestWriteStateForMigrationWithForcePushClient(t *testing.T) {
 					"terraform_version": version.Version,
 					"outputs":           map[string]interface{}{"foo": map[string]interface{}{"type": string("string"), "value": string("bar")}},
 					"resources":         []interface{}{},
+					"check_results":     nil,
 				},
 			},
 			force: true,
@@ -519,6 +662,7 @@ func TestWriteStateForMigrationWithForcePushClient(t *testing.T) {
 					"terraform_version": version.Version,
 					"outputs":           map[string]interface{}{"foo": map[string]interface{}{"type": string("string"), "value": string("bar")}},
 					"resources":         []interface{}{},
+					"check_results":     nil,
 				},
 			},
 			force: true,
@@ -551,43 +695,45 @@ func TestWriteStateForMigrationWithForcePushClient(t *testing.T) {
 	logIdx := 0
 
 	for _, tc := range testCases {
-		// Always reset client to not be force pushing
-		mockClient.force = false
-		sf := tc.stateFile(mgr)
-		err := mgr.WriteStateForMigration(sf, tc.force)
-		shouldError := tc.expectedError != ""
+		t.Run(tc.name, func(t *testing.T) {
+			// Always reset client to not be force pushing
+			mockClient.force = false
+			sf := tc.stateFile(mgr)
+			err := mgr.WriteStateForMigration(sf, tc.force)
+			shouldError := tc.expectedError != ""
 
-		// If we are expecting and error check it and move on
-		if shouldError {
-			if err == nil {
-				t.Fatalf("test case %q should have failed with error %q", tc.name, tc.expectedError)
-			} else if err.Error() != tc.expectedError {
-				t.Fatalf("test case %q expected error %q but got %q", tc.name, tc.expectedError, err)
+			// If we are expecting and error check it and move on
+			if shouldError {
+				if err == nil {
+					t.Fatalf("test case %q should have failed with error %q", tc.name, tc.expectedError)
+				} else if err.Error() != tc.expectedError {
+					t.Fatalf("test case %q expected error %q but got %q", tc.name, tc.expectedError, err)
+				}
+				return
 			}
-			continue
-		}
 
-		if err != nil {
-			t.Fatalf("test case %q failed: %v", tc.name, err)
-		}
+			if err != nil {
+				t.Fatalf("test case %q failed: %v", tc.name, err)
+			}
 
-		if tc.force && !mockClient.force {
-			t.Fatalf("test case %q should have enabled force push", tc.name)
-		}
+			if tc.force && !mockClient.force {
+				t.Fatalf("test case %q should have enabled force push", tc.name)
+			}
 
-		// At this point we should just do a normal write and persist
-		// as would happen from the CLI
-		mgr.WriteState(mgr.State())
-		mgr.PersistState()
+			// At this point we should just do a normal write and persist
+			// as would happen from the CLI
+			mgr.WriteState(mgr.State())
+			mgr.PersistState(nil)
 
-		if logIdx >= len(mockClient.log) {
-			t.Fatalf("request lock and index are out of sync on %q: idx=%d len=%d", tc.name, logIdx, len(mockClient.log))
-		}
-		loggedRequest := mockClient.log[logIdx]
-		logIdx++
-		if diff := cmp.Diff(tc.expectedRequest, loggedRequest); len(diff) > 0 {
-			t.Fatalf("incorrect client requests for %q:\n%s", tc.name, diff)
-		}
+			if logIdx >= len(mockClient.log) {
+				t.Fatalf("request lock and index are out of sync on %q: idx=%d len=%d", tc.name, logIdx, len(mockClient.log))
+			}
+			loggedRequest := mockClient.log[logIdx]
+			logIdx++
+			if diff := cmp.Diff(tc.expectedRequest, loggedRequest); len(diff) > 0 {
+				t.Fatalf("incorrect client requests for %q:\n%s", tc.name, diff)
+			}
+		})
 	}
 
 	logCnt := len(mockClient.log)
