@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package terraform
 
 import (
@@ -5,14 +8,14 @@ import (
 	"log"
 
 	"github.com/hashicorp/hcl/v2"
+	"github.com/zclconf/go-cty/cty"
+
 	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/configs"
 	"github.com/hashicorp/terraform/internal/dag"
 	"github.com/hashicorp/terraform/internal/instances"
 	"github.com/hashicorp/terraform/internal/lang"
 	"github.com/hashicorp/terraform/internal/tfdiags"
-	"github.com/zclconf/go-cty/cty"
-	"github.com/zclconf/go-cty/cty/convert"
 )
 
 // nodeExpandModuleVariable is the placeholder for an variable that has not yet had
@@ -22,6 +25,14 @@ type nodeExpandModuleVariable struct {
 	Module addrs.Module
 	Config *configs.Variable
 	Expr   hcl.Expression
+
+	// Planning must be set to true when building a planning graph, and must be
+	// false when building an apply graph.
+	Planning bool
+
+	// DestroyApply must be set to true when planning or applying a destroy
+	// operation, and false otherwise.
+	DestroyApply bool
 }
 
 var (
@@ -41,16 +52,40 @@ func (n *nodeExpandModuleVariable) temporaryValue() bool {
 
 func (n *nodeExpandModuleVariable) DynamicExpand(ctx EvalContext) (*Graph, error) {
 	var g Graph
+
+	// If this variable has preconditions, we need to report these checks now.
+	//
+	// We should only do this during planning as the apply phase starts with
+	// all the same checkable objects that were registered during the plan.
+	var checkableAddrs addrs.Set[addrs.Checkable]
+	if n.Planning {
+		if checkState := ctx.Checks(); checkState.ConfigHasChecks(n.Addr.InModule(n.Module)) {
+			checkableAddrs = addrs.MakeSet[addrs.Checkable]()
+		}
+	}
+
 	expander := ctx.InstanceExpander()
 	for _, module := range expander.ExpandModule(n.Module) {
+		addr := n.Addr.Absolute(module)
+		if checkableAddrs != nil {
+			checkableAddrs.Add(addr)
+		}
+
 		o := &nodeModuleVariable{
-			Addr:           n.Addr.Absolute(module),
+			Addr:           addr,
 			Config:         n.Config,
 			Expr:           n.Expr,
 			ModuleInstance: module,
+			DestroyApply:   n.DestroyApply,
 		}
 		g.Add(o)
 	}
+	addRootNodeToGraph(&g)
+
+	if checkableAddrs != nil {
+		ctx.Checks().ReportCheckableObjects(n.Addr.InModule(n.Module), checkableAddrs)
+	}
+
 	return &g, nil
 }
 
@@ -87,7 +122,7 @@ func (n *nodeExpandModuleVariable) References() []*addrs.Reference {
 	// where our associated variable was declared, which is correct because
 	// our value expression is assigned within a "module" block in the parent
 	// module.
-	refs, _ := lang.ReferencesInExpr(n.Expr)
+	refs, _ := lang.ReferencesInExpr(addrs.ParseRef, n.Expr)
 	return refs
 }
 
@@ -110,6 +145,10 @@ type nodeModuleVariable struct {
 	// ModuleInstance in order to create the appropriate context for evaluating
 	// ModuleCallArguments, ex. so count.index and each.key can resolve
 	ModuleInstance addrs.ModuleInstance
+
+	// DestroyApply must be set to true when applying a destroy operation and
+	// false otherwise.
+	DestroyApply bool
 }
 
 // Ensure that we are implementing all of the interfaces we think we are
@@ -143,37 +182,36 @@ func (n *nodeModuleVariable) ModulePath() addrs.Module {
 
 // GraphNodeExecutable
 func (n *nodeModuleVariable) Execute(ctx EvalContext, op walkOperation) (diags tfdiags.Diagnostics) {
-	// If we have no value, do nothing
-	if n.Expr == nil {
-		return nil
-	}
+	log.Printf("[TRACE] nodeModuleVariable: evaluating %s", n.Addr)
 
-	// Otherwise, interpolate the value of this variable and set it
-	// within the variables mapping.
-	var vals map[string]cty.Value
+	var val cty.Value
 	var err error
 
 	switch op {
 	case walkValidate:
-		vals, err = n.evalModuleCallArgument(ctx, true)
+		val, err = n.evalModuleVariable(ctx, true)
 		diags = diags.Append(err)
-		if diags.HasErrors() {
-			return diags
-		}
 	default:
-		vals, err = n.evalModuleCallArgument(ctx, false)
+		val, err = n.evalModuleVariable(ctx, false)
 		diags = diags.Append(err)
-		if diags.HasErrors() {
-			return diags
-		}
+	}
+	if diags.HasErrors() {
+		return diags
 	}
 
 	// Set values for arguments of a child module call, for later retrieval
 	// during expression evaluation.
 	_, call := n.Addr.Module.CallInstance()
-	ctx.SetModuleCallArguments(call, vals)
+	ctx.SetModuleCallArgument(call, n.Addr.Variable, val)
 
-	return evalVariableValidations(n.Addr, n.Config, n.Expr, ctx)
+	// Skip evalVariableValidations during destroy operations. We still want
+	// to evaluate the variable in case it is used to initialise providers
+	// or something downstream but we don't need to report on the success
+	// or failure of any validations for destroy operations.
+	if !n.DestroyApply {
+		diags = diags.Append(evalVariableValidations(n.Addr, n.Config, n.Expr, ctx))
+	}
+	return diags
 }
 
 // dag.GraphNodeDotter impl.
@@ -187,7 +225,7 @@ func (n *nodeModuleVariable) DotNode(name string, opts *dag.DotOpts) *dag.DotNod
 	}
 }
 
-// evalModuleCallArgument produces the value for a particular variable as will
+// evalModuleVariable produces the value for a particular variable as will
 // be used by a child module instance.
 //
 // The result is written into a map, with its key set to the local name of the
@@ -199,100 +237,54 @@ func (n *nodeModuleVariable) DotNode(name string, opts *dag.DotOpts) *dag.DotNod
 // validateOnly indicates that this evaluation is only for config
 // validation, and we will not have any expansion module instance
 // repetition data.
-func (n *nodeModuleVariable) evalModuleCallArgument(ctx EvalContext, validateOnly bool) (map[string]cty.Value, error) {
-	name := n.Addr.Variable.Name
-	expr := n.Expr
+func (n *nodeModuleVariable) evalModuleVariable(ctx EvalContext, validateOnly bool) (cty.Value, error) {
+	var diags tfdiags.Diagnostics
+	var givenVal cty.Value
+	var errSourceRange tfdiags.SourceRange
+	if expr := n.Expr; expr != nil {
+		var moduleInstanceRepetitionData instances.RepetitionData
 
-	if expr == nil {
-		// Should never happen, but we'll bail out early here rather than
-		// crash in case it does. We set no value at all in this case,
-		// making a subsequent call to EvalContext.SetModuleCallArguments
-		// a no-op.
-		log.Printf("[ERROR] attempt to evaluate %s with nil expression", n.Addr.String())
-		return nil, nil
-	}
+		switch {
+		case validateOnly:
+			// the instance expander does not track unknown expansion values, so we
+			// have to assume all RepetitionData is unknown.
+			moduleInstanceRepetitionData = instances.RepetitionData{
+				CountIndex: cty.UnknownVal(cty.Number),
+				EachKey:    cty.UnknownVal(cty.String),
+				EachValue:  cty.DynamicVal,
+			}
 
-	var moduleInstanceRepetitionData instances.RepetitionData
-
-	switch {
-	case validateOnly:
-		// the instance expander does not track unknown expansion values, so we
-		// have to assume all RepetitionData is unknown.
-		moduleInstanceRepetitionData = instances.RepetitionData{
-			CountIndex: cty.UnknownVal(cty.Number),
-			EachKey:    cty.UnknownVal(cty.String),
-			EachValue:  cty.DynamicVal,
+		default:
+			// Get the repetition data for this module instance,
+			// so we can create the appropriate scope for evaluating our expression
+			moduleInstanceRepetitionData = ctx.InstanceExpander().GetModuleInstanceRepetitionData(n.ModuleInstance)
 		}
 
-	default:
-		// Get the repetition data for this module instance,
-		// so we can create the appropriate scope for evaluating our expression
-		moduleInstanceRepetitionData = ctx.InstanceExpander().GetModuleInstanceRepetitionData(n.ModuleInstance)
+		scope := ctx.EvaluationScope(nil, nil, moduleInstanceRepetitionData)
+		val, moreDiags := scope.EvalExpr(expr, cty.DynamicPseudoType)
+		diags = diags.Append(moreDiags)
+		if moreDiags.HasErrors() {
+			return cty.DynamicVal, diags.ErrWithWarnings()
+		}
+		givenVal = val
+		errSourceRange = tfdiags.SourceRangeFromHCL(expr.Range())
+	} else {
+		// We'll use cty.NilVal to represent the variable not being set at all.
+		givenVal = cty.NilVal
+		errSourceRange = tfdiags.SourceRangeFromHCL(n.Config.DeclRange) // we use the declaration range as a fallback for an undefined variable
 	}
 
-	scope := ctx.EvaluationScope(nil, moduleInstanceRepetitionData)
-	val, diags := scope.EvalExpr(expr, cty.DynamicPseudoType)
-
-	// FIXME: This check is only necessary for v1.1, and is will never be
-	// present in the v1.2 branch.
-	//
-	// Default values are currently being handled in two places, the graph
-	// transformation where a synthetic default expression is created if there
-	// was no input expression, and in the evaluator when a reference to the
-	// variable is evaluated. Unfortunately neither of these covers the case
-	// where a non-nullable variable default needs to be checked within a
-	// validation statement
-	//
-	// Rather than try and fix the overall variable handling here, which runs
-	// the risk of encountering more unexpected behavior, we are going to fixup
-	// this one case to ensure a null value doesn't continue to slip into
-	// validation. A more extensive refactoring of variable handling has been
-	// completed in v1.2, and this code will only be relevant for the v1.1
-	// branch.
-	if !diags.HasErrors() && val.IsNull() &&
-		!n.Config.Nullable &&
-		n.Config.Default != cty.NilVal && !n.Config.Default.IsNull() {
-		// replace the evaluated value with the actual default
-		val = n.Config.Default
+	// We construct a synthetic InputValue here to pretend as if this were
+	// a root module variable set from outside, just as a convenience so we
+	// can reuse the InputValue type for this.
+	rawVal := &InputValue{
+		Value:       givenVal,
+		SourceType:  ValueFromConfig,
+		SourceRange: errSourceRange,
 	}
 
-	// We intentionally passed DynamicPseudoType to EvalExpr above because
-	// now we can do our own local type conversion and produce an error message
-	// with better context if it fails.
-	var convErr error
-	val, convErr = convert.Convert(val, n.Config.ConstraintType)
-	if convErr != nil {
-		diags = diags.Append(&hcl.Diagnostic{
-			Severity: hcl.DiagError,
-			Summary:  "Invalid value for module argument",
-			Detail: fmt.Sprintf(
-				"The given value is not suitable for child module variable %q defined at %s: %s.",
-				name, n.Config.DeclRange.String(), convErr,
-			),
-			Subject: expr.Range().Ptr(),
-		})
-		// We'll return a placeholder unknown value to avoid producing
-		// redundant downstream errors.
-		val = cty.UnknownVal(n.Config.Type)
-	}
+	finalVal, moreDiags := prepareFinalInputVariableValue(n.Addr, rawVal, n.Config)
+	diags = diags.Append(moreDiags)
 
-	// If there is no default, we have to ensure that a null value is allowed
-	// for this variable.
-	if n.Config.Default == cty.NilVal && !n.Config.Nullable && val.IsNull() {
-		// The value cannot be null, and there is no configured default.
-		diags = diags.Append(&hcl.Diagnostic{
-			Severity: hcl.DiagError,
-			Summary:  `Invalid variable value`,
-			Detail:   fmt.Sprintf(`The variable %q is required, but the given value is null.`, n.Addr),
-			Subject:  &n.Config.DeclRange,
-		})
-		// Stub out our return value so that the semantic checker doesn't
-		// produce redundant downstream errors.
-		val = cty.UnknownVal(n.Config.Type)
-	}
-
-	vals := make(map[string]cty.Value)
-	vals[name] = val
-
-	return vals, diags.ErrWithWarnings()
+	return finalVal, diags.ErrWithWarnings()
 }
