@@ -1,10 +1,12 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package terraform
 
 import (
 	"context"
 	"flag"
 	"io"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"strings"
@@ -58,11 +60,14 @@ func testModuleWithSnapshot(t *testing.T, name string) (*configs.Config, *config
 	// change its interface at this late stage.
 	loader, _ := configload.NewLoaderForTests(t)
 
+	// We need to be able to exercise experimental features in our integration tests.
+	loader.AllowLanguageExperiments(true)
+
 	// Test modules usually do not refer to remote sources, and for local
 	// sources only this ultimately just records all of the module paths
 	// in a JSON file so that we can load them below.
-	inst := initwd.NewModuleInstaller(loader.ModulesDir(), registry.NewClient(nil, nil))
-	_, instDiags := inst.InstallModules(context.Background(), dir, true, initwd.ModuleInstallHooksImpl{})
+	inst := initwd.NewModuleInstaller(loader.ModulesDir(), loader, registry.NewClient(nil, nil))
+	_, instDiags := inst.InstallModules(context.Background(), dir, "tests", true, false, initwd.ModuleInstallHooksImpl{})
 	if instDiags.HasErrors() {
 		t.Fatal(instDiags.Err())
 	}
@@ -86,11 +91,7 @@ func testModuleWithSnapshot(t *testing.T, name string) (*configs.Config, *config
 func testModuleInline(t *testing.T, sources map[string]string) *configs.Config {
 	t.Helper()
 
-	cfgPath, err := ioutil.TempDir("", "tf-test")
-	if err != nil {
-		t.Errorf("Error creating temporary directory for config: %s", err)
-	}
-	defer os.RemoveAll(cfgPath)
+	cfgPath := t.TempDir()
 
 	for path, configStr := range sources {
 		dir := filepath.Dir(path)
@@ -116,11 +117,14 @@ func testModuleInline(t *testing.T, sources map[string]string) *configs.Config {
 	loader, cleanup := configload.NewLoaderForTests(t)
 	defer cleanup()
 
+	// We need to be able to exercise experimental features in our integration tests.
+	loader.AllowLanguageExperiments(true)
+
 	// Test modules usually do not refer to remote sources, and for local
 	// sources only this ultimately just records all of the module paths
 	// in a JSON file so that we can load them below.
-	inst := initwd.NewModuleInstaller(loader.ModulesDir(), registry.NewClient(nil, nil))
-	_, instDiags := inst.InstallModules(context.Background(), cfgPath, true, initwd.ModuleInstallHooksImpl{})
+	inst := initwd.NewModuleInstaller(loader.ModulesDir(), loader, registry.NewClient(nil, nil))
+	_, instDiags := inst.InstallModules(context.Background(), cfgPath, "tests", true, false, initwd.ModuleInstallHooksImpl{})
 	if instDiags.HasErrors() {
 		t.Fatal(instDiags.Err())
 	}
@@ -131,7 +135,7 @@ func testModuleInline(t *testing.T, sources map[string]string) *configs.Config {
 		t.Fatalf("failed to refresh modules after installation: %s", err)
 	}
 
-	config, diags := loader.LoadConfig(cfgPath)
+	config, diags := loader.LoadConfigWithTests(cfgPath, "tests")
 	if diags.HasErrors() {
 		t.Fatal(diags.Error())
 	}
@@ -166,32 +170,33 @@ func testSetResourceInstanceTainted(module *states.Module, resource, attrsJson,
 }
 
 func testProviderFuncFixed(rp providers.Interface) providers.Factory {
-	return func() (providers.Interface, error) {
-		if p, ok := rp.(*MockProvider); ok {
-			// make sure none of the methods were "called" on this new instance
-			p.GetProviderSchemaCalled = false
-			p.ValidateProviderConfigCalled = false
-			p.ValidateResourceConfigCalled = false
-			p.ValidateDataResourceConfigCalled = false
-			p.UpgradeResourceStateCalled = false
-			p.ConfigureProviderCalled = false
-			p.StopCalled = false
-			p.ReadResourceCalled = false
-			p.PlanResourceChangeCalled = false
-			p.ApplyResourceChangeCalled = false
-			p.ImportResourceStateCalled = false
-			p.ReadDataSourceCalled = false
-			p.CloseCalled = false
-		}
+	if p, ok := rp.(*MockProvider); ok {
+		// make sure none of the methods were "called" on this new instance
+		p.GetProviderSchemaCalled = false
+		p.ValidateProviderConfigCalled = false
+		p.ValidateResourceConfigCalled = false
+		p.ValidateDataResourceConfigCalled = false
+		p.UpgradeResourceStateCalled = false
+		p.ConfigureProviderCalled = false
+		p.StopCalled = false
+		p.ReadResourceCalled = false
+		p.PlanResourceChangeCalled = false
+		p.ApplyResourceChangeCalled = false
+		p.ImportResourceStateCalled = false
+		p.ReadDataSourceCalled = false
+		p.CloseCalled = false
+	}
 
+	return func() (providers.Interface, error) {
 		return rp, nil
 	}
 }
 
 func testProvisionerFuncFixed(rp *MockProvisioner) provisioners.Factory {
+	// make sure this provisioner has has not been closed
+	rp.CloseCalled = false
+
 	return func() (provisioners.Interface, error) {
-		// make sure this provisioner has has not been closed
-		rp.CloseCalled = false
 		return rp, nil
 	}
 }
@@ -228,6 +233,14 @@ func mustProviderConfig(s string) addrs.AbsProviderConfig {
 	return p
 }
 
+func mustReference(s string) *addrs.Reference {
+	p, diags := addrs.ParseRefStr(s)
+	if diags.HasErrors() {
+		panic(diags.Err())
+	}
+	return p
+}
+
 // HookRecordApplyOrder is a test hook that records the order of applies
 // by recording the PreApply event.
 type HookRecordApplyOrder struct {
