@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package terraform
 
 import (
@@ -6,6 +9,7 @@ import (
 	"testing"
 
 	"github.com/hashicorp/terraform/internal/addrs"
+	"github.com/hashicorp/terraform/internal/dag"
 	"github.com/hashicorp/terraform/internal/plans"
 	"github.com/hashicorp/terraform/internal/states"
 )
@@ -46,13 +50,25 @@ func cbdTestSteps(steps []GraphTransformer) []GraphTransformer {
 		panic("CBDEdgeTransformer not found")
 	}
 
-	return steps[:i+1]
+	// re-add the root node so we have a valid graph for a walk, then reduce
+	// the graph for less output
+	steps = append(steps[:i+1], &CloseRootModuleTransformer{})
+	steps = append(steps, &TransitiveReductionTransformer{})
+
+	return steps
 }
 
 // remove extra nodes for easier test comparisons
 func filterInstances(g *Graph) *Graph {
 	for _, v := range g.Vertices() {
 		if _, ok := v.(GraphNodeResourceInstance); !ok {
+			// connect around the node to remove it without breaking deps
+			for _, down := range g.DownEdges(v) {
+				for _, up := range g.UpEdges(v) {
+					g.Connect(dag.BasicEdge(up, down))
+				}
+			}
+
 			g.Remove(v)
 		}
 
@@ -105,7 +121,6 @@ func TestCBDEdgeTransformer(t *testing.T) {
 	expected := regexp.MustCompile(strings.TrimSpace(`
 (?m)test_object.A
 test_object.A \(destroy deposed \w+\)
-  test_object.A
   test_object.B
 test_object.B
   test_object.A
@@ -178,11 +193,9 @@ func TestCBDEdgeTransformerMulti(t *testing.T) {
 	expected := regexp.MustCompile(strings.TrimSpace(`
 (?m)test_object.A
 test_object.A \(destroy deposed \w+\)
-  test_object.A
   test_object.C
 test_object.B
 test_object.B \(destroy deposed \w+\)
-  test_object.B
   test_object.C
 test_object.C
   test_object.A
@@ -253,7 +266,6 @@ func TestCBDEdgeTransformer_depNonCBDCount(t *testing.T) {
 	expected := regexp.MustCompile(strings.TrimSpace(`
 (?m)test_object.A
 test_object.A \(destroy deposed \w+\)
-  test_object.A
   test_object.B\[0\]
   test_object.B\[1\]
 test_object.B\[0\]
@@ -339,12 +351,10 @@ func TestCBDEdgeTransformer_depNonCBDCountBoth(t *testing.T) {
 	expected := regexp.MustCompile(strings.TrimSpace(`
 test_object.A\[0\]
 test_object.A\[0\] \(destroy deposed \w+\)
-  test_object.A\[0\]
   test_object.B\[0\]
   test_object.B\[1\]
 test_object.A\[1\]
 test_object.A\[1\] \(destroy deposed \w+\)
-  test_object.A\[1\]
   test_object.B\[0\]
   test_object.B\[1\]
 test_object.B\[0\]
